@@ -25,10 +25,10 @@
  * using intel's AES instruction set. 
  */
 
-#include <gnutls_errors.h>
-#include <gnutls_int.h>
+#include "errors.h"
+#include "gnutls_int.h"
 #include <gnutls/crypto.h>
-#include <gnutls_errors.h>
+#include "errors.h"
 #include <aes-x86.h>
 #include <sha-x86.h>
 #include <x86-common.h>
@@ -46,7 +46,7 @@ __attribute__((visibility("hidden")))
 #elif defined(__SUNPRO_C)
 __hidden
 #endif
-unsigned int _gnutls_x86_cpuid_s[3];
+unsigned int _gnutls_x86_cpuid_s[4];
 
 #ifndef bit_PCLMUL
 # define bit_PCLMUL 0x2
@@ -103,6 +103,7 @@ static void capabilities_to_intel_cpuid(unsigned capabilities)
 			    ("SSSE3 acceleration requested but not available\n");
 		}
 	}
+
 	if (capabilities & INTEL_PCLMUL) {
 		if (b & bit_PCLMUL) {
 			_gnutls_x86_cpuid_s[1] |= bit_PCLMUL;
@@ -111,6 +112,7 @@ static void capabilities_to_intel_cpuid(unsigned capabilities)
 			    ("PCLMUL acceleration requested but not available\n");
 		}
 	}
+
 }
 
 static unsigned check_optimized_aes(void)
@@ -275,7 +277,10 @@ void register_x86_padlock_crypto(unsigned capabilities)
 			gnutls_assert();
 		}
 #endif
+	} else {
+		_gnutls_priority_update_non_aesni();
 	}
+
 #ifdef HAVE_LIBNETTLE
 	phe = check_phe(edx);
 
@@ -644,6 +649,8 @@ void register_x86_intel_crypto(unsigned capabilities)
 				gnutls_assert();
 			}
 		}
+	} else {
+		_gnutls_priority_update_non_aesni();
 	}
 
 	return;
@@ -654,7 +661,7 @@ void register_x86_crypto(void)
 {
 	unsigned capabilities = 0;
 	char *p;
-	p = secure_getenv("GNUTLS_CPUID_OVERRIDE");
+	p = getenv("GNUTLS_CPUID_OVERRIDE");
 	if (p) {
 		capabilities = strtol(p, NULL, 0);
 	}
