@@ -20,9 +20,9 @@
  *
  */
 
-#include <gnutls_int.h>
+#include "gnutls_int.h"
 #include <algorithms.h>
-#include <gnutls_errors.h>
+#include "errors.h"
 #include <x509/common.h>
 
 
@@ -32,8 +32,9 @@
  * protecting communications" by Hugo Krawczyk - CRYPTO 2001
  *
  * On update, make sure to update MAX_CIPHER_BLOCK_SIZE and MAX_CIPHER_KEY_SIZE
- * as well. If any ciphers are removed, modify the is_legacy() functions
- * in priority.c.
+ * as well. If any ciphers are removed, remove them from the back-end but
+ * keep them in that list to allow backwards compatibility with applications
+ * that specify them (they will be a no-op).
  */
 static const cipher_entry_st algorithms[] = {
 	{ .name = "AES-256-CBC",
@@ -155,8 +156,9 @@ static const cipher_entry_st algorithms[] = {
 	  .blocksize = 64,
 	  .keysize = 32,
 	  .type = CIPHER_AEAD,
-	  .implicit_iv = 4,
-	  .explicit_iv = 8,
+	  .implicit_iv = 12,
+	  .explicit_iv = 0,
+	  .xor_nonce = 1,
 	  .cipher_iv = 12,
 	  .tagsize = 16
 	},
