@@ -24,29 +24,29 @@
  * and are common to RSA and DHE key exchange, are in this file.
  */
 
-#include <gnutls_int.h>
-#include "gnutls_auth.h"
-#include "gnutls_errors.h"
+#include "gnutls_int.h"
+#include "auth.h"
+#include "errors.h"
 #include <auth/cert.h>
-#include "gnutls_dh.h"
-#include "gnutls_num.h"
+#include "dh.h"
+#include "num.h"
 #include "libtasn1.h"
-#include "gnutls_datum.h"
+#include "datum.h"
 #include "ext/signature.h"
-#include <gnutls_pk.h>
+#include <pk.h>
 #include <algorithms.h>
-#include <gnutls_global.h>
-#include <gnutls_record.h>
-#include <gnutls_sig.h>
-#include <gnutls_state.h>
-#include <gnutls_pk.h>
-#include <gnutls_x509.h>
+#include <global.h>
+#include <record.h>
+#include <tls-sig.h>
+#include <state.h>
+#include <pk.h>
+#include <x509.h>
 #include <x509/verify-high.h>
 #include <gnutls/abstract.h>
 #include "debug.h"
 
 #ifdef ENABLE_OPENPGP
-#include "openpgp/gnutls_openpgp.h"
+#include "openpgp/openpgp.h"
 
 static gnutls_privkey_t alloc_and_load_pgp_key(const
 					       gnutls_openpgp_privkey_t
@@ -204,7 +204,8 @@ static int cert_get_issuer_dn(gnutls_pcert_st * cert, gnutls_datum_t * odn)
  * CAs and sign algorithms supported by the peer server.
  */
 static int
-find_x509_cert(const gnutls_certificate_credentials_t cred,
+find_x509_client_cert(gnutls_session_t session,
+	       const gnutls_certificate_credentials_t cred,
 	       uint8_t * _data, size_t _data_size,
 	       const gnutls_pk_algorithm_t * pk_algos,
 	       int pk_algos_length, int *indx)
@@ -221,9 +222,10 @@ find_x509_cert(const gnutls_certificate_credentials_t cred,
 	/* If peer doesn't send any issuers and we have a single certificate
 	 * then send that one.
 	 */
-	if (data_size == 0 && cred->ncerts == 1) {
-		*indx = 0;
-		return 0;
+	if (cred->ncerts == 1 &&
+		(data_size == 0 || (session->internals.flags & GNUTLS_FORCE_CLIENT_CERT))) {
+			*indx = 0;
+			return 0;
 	}
 
 	do {
@@ -404,32 +406,6 @@ get_issuers(gnutls_session_t session,
 	return 0;
 }
 
-static void st_to_st2(gnutls_retr2_st * st2, gnutls_retr_st * st)
-{
-	st2->cert_type = st->type;
-	if (st->type == GNUTLS_CRT_OPENPGP) {
-		st2->key_type = GNUTLS_PRIVKEY_OPENPGP;
-	} else {
-		st2->key_type = GNUTLS_PRIVKEY_X509;
-	}
-	st2->ncerts = st->ncerts;
-	st2->deinit_all = st->deinit_all;
-
-	switch (st2->cert_type) {
-	case GNUTLS_CRT_OPENPGP:
-		st2->cert.pgp = st->cert.pgp;
-		st2->key.pgp = st->key.pgp;
-		break;
-	case GNUTLS_CRT_X509:
-		st2->cert.x509 = st->cert.x509;
-		st2->key.x509 = st->key.x509;
-		break;
-	default:
-		return;
-	}
-
-}
-
 /* Calls the client get callback.
  */
 static int
@@ -486,31 +462,8 @@ call_get_cert_callback(gnutls_session_t session,
 					    pk_algos_length, &st2);
 
 	} else {		/* compatibility mode */
-		gnutls_retr_st st;
-		memset(&st, 0, sizeof(st));
-		if (session->security_parameters.entity == GNUTLS_SERVER) {
-			if (cred->server_get_cert_callback == NULL) {
-				gnutls_assert();
-				return GNUTLS_E_INTERNAL_ERROR;
-			}
-			ret = cred->server_get_cert_callback(session, &st);
-			if (ret >= 0)
-				st_to_st2(&st2, &st);
-		} else {	/* CLIENT */
-
-			if (cred->client_get_cert_callback == NULL) {
-				gnutls_assert();
-				return GNUTLS_E_INTERNAL_ERROR;
-			}
-			ret = cred->client_get_cert_callback(session,
-							     issuers_dn,
-							     issuers_dn_length,
-							     pk_algos,
-							     pk_algos_length,
-							     &st);
-			if (ret >= 0)
-				st_to_st2(&st2, &st);
-		}
+		gnutls_assert();
+		return GNUTLS_E_INTERNAL_ERROR;
 	}
 
 	if (ret < 0) {
@@ -655,8 +608,7 @@ select_client_cert(gnutls_session_t session,
 		return GNUTLS_E_INSUFFICIENT_CREDENTIALS;
 	}
 
-	if (cred->client_get_cert_callback != NULL
-	    || cred->get_cert_callback != NULL
+	if (cred->get_cert_callback != NULL
 	    || cred->get_cert_callback2 != NULL) {
 
 		/* use a callback to get certificate 
@@ -704,7 +656,7 @@ select_client_cert(gnutls_session_t session,
 
 		if (session->security_parameters.cert_type == GNUTLS_CRT_X509)
 			result =
-			    find_x509_cert(cred, _data, _data_size,
+			    find_x509_client_cert(session, cred, _data, _data_size,
 					   pk_algos, pk_algos_length, &indx);
 #ifdef ENABLE_OPENPGP
 		else if (session->security_parameters.cert_type ==
@@ -2010,7 +1962,7 @@ _gnutls_server_select_cert(gnutls_session_t session,
 	/* If the callback which retrieves certificate has been set,
 	 * use it and leave.
 	 */
-	if (cred->server_get_cert_callback || cred->get_cert_callback
+	if (cred->get_cert_callback
 	    || cred->get_cert_callback2) {
 		ret = call_get_cert_callback(session, NULL, 0, NULL, 0);
 		if (ret < 0)
