@@ -25,20 +25,20 @@
  */
 
 #include "gnutls_int.h"
-#include "gnutls_auth.h"
-#include "gnutls_errors.h"
-#include "gnutls_dh.h"
-#include "gnutls_num.h"
-#include "gnutls_datum.h"
+#include "auth.h"
+#include "errors.h"
+#include "dh.h"
+#include "num.h"
+#include "datum.h"
 #include <auth/cert.h>
-#include <gnutls_pk.h>
+#include <pk.h>
 #include <algorithms.h>
-#include <gnutls_global.h>
+#include <global.h>
 #include "debug.h"
-#include <gnutls_sig.h>
-#include <gnutls_x509.h>
+#include <tls-sig.h>
+#include <x509.h>
 #include <random.h>
-#include <gnutls_mpi.h>
+#include <mpi.h>
 #include <abstract_int.h>
 #include <auth/rsa_common.h>
 
@@ -62,6 +62,25 @@ const mod_auth_st rsa_auth_struct = {
 	_gnutls_proc_cert_cert_req	/* proc server cert request */
 };
 
+static
+int check_key_usage_for_enc(gnutls_session_t session, unsigned key_usage)
+{
+	if (key_usage != 0) {
+		if (!(key_usage & GNUTLS_KEY_KEY_ENCIPHERMENT) && !(key_usage & GNUTLS_KEY_KEY_AGREEMENT)) {
+			gnutls_assert();
+			if (session->internals.priorities.allow_key_usage_violation == 0) {
+				_gnutls_audit_log(session,
+					  "Peer's certificate does not allow encryption. Key usage violation detected.\n");
+				return GNUTLS_E_KEY_USAGE_VIOLATION;
+			} else {
+				_gnutls_audit_log(session,
+					  "Peer's certificate does not allow encryption. Key usage violation detected (ignored).\n");
+			}
+		}
+	}
+	return 0;
+}
+
 /* This function reads the RSA parameters from peer's certificate;
  */
 int
@@ -70,6 +89,7 @@ _gnutls_get_public_rsa_params(gnutls_session_t session,
 {
 	int ret;
 	cert_auth_info_t info;
+	unsigned key_usage;
 	gnutls_pcert_st peer_cert;
 
 	/* normal non export case */
@@ -91,6 +111,14 @@ _gnutls_get_public_rsa_params(gnutls_session_t session,
 		return ret;
 	}
 
+	gnutls_pubkey_get_key_usage(peer_cert.pubkey, &key_usage);
+
+	ret = check_key_usage_for_enc(session, key_usage);
+	if (ret < 0) {
+		gnutls_assert();
+		goto cleanup2;
+	}
+
 	gnutls_pk_params_init(params);
 
 	ret = _gnutls_pubkey_get_mpis(peer_cert.pubkey, params);
@@ -115,8 +143,9 @@ proc_rsa_client_kx(gnutls_session_t session, uint8_t * data,
 	gnutls_datum_t plaintext;
 	gnutls_datum_t ciphertext;
 	int ret, dsize;
-	int randomize_key = 0;
+	int use_rnd_key = 0;
 	ssize_t data_size = _data_size;
+	gnutls_datum_t rndkey = {NULL, 0};
 
 	if (get_num_version(session) == GNUTLS_SSL3) {
 		/* SSL 3.0 
@@ -137,6 +166,22 @@ proc_rsa_client_kx(gnutls_session_t session, uint8_t * data,
 		ciphertext.size = dsize;
 	}
 
+	rndkey.size = GNUTLS_MASTER_SIZE;
+	rndkey.data = gnutls_malloc(rndkey.size);
+	if (rndkey.data == NULL) {
+		gnutls_assert();
+		return GNUTLS_E_MEMORY_ERROR;
+	}
+
+	/* we do not need strong random numbers here.
+	 */
+	ret = _gnutls_rnd(GNUTLS_RND_NONCE, rndkey.data,
+			  rndkey.size);
+	if (ret < 0) {
+		gnutls_assert();
+		goto cleanup;
+	}
+
 	ret =
 	    gnutls_privkey_decrypt_data(session->internals.selected_key, 0,
 					&ciphertext, &plaintext);
@@ -146,10 +191,8 @@ proc_rsa_client_kx(gnutls_session_t session, uint8_t * data,
 		 * the peer. Just use a random key. (in order to avoid
 		 * attack against pkcs-1 formating).
 		 */
-		gnutls_assert();
-		_gnutls_audit_log(session,
-				  "auth_rsa: Possible PKCS #1 format attack\n");
-		randomize_key = 1;
+		_gnutls_debug_log("auth_rsa: Possible PKCS #1 format attack\n");
+		use_rnd_key = 1;
 	} else {
 		/* If the secret was properly formatted, then
 		 * check the version number.
@@ -165,31 +208,14 @@ proc_rsa_client_kx(gnutls_session_t session, uint8_t * data,
 			 * "Attacking RSA-based sessions in SSL/TLS" by Vlastimil Klima,
 			 * Ondej Pokorny and Tomas Rosa.
 			 */
-			gnutls_assert();
-			_gnutls_audit_log
-			    (session,
-			     "auth_rsa: Possible PKCS #1 version check format attack\n");
+			_gnutls_debug_log("auth_rsa: Possible PKCS #1 version check format attack\n");
 		}
 	}
 
-	if (randomize_key != 0) {
-		session->key.key.size = GNUTLS_MASTER_SIZE;
-		session->key.key.data =
-		    gnutls_malloc(session->key.key.size);
-		if (session->key.key.data == NULL) {
-			gnutls_assert();
-			return GNUTLS_E_MEMORY_ERROR;
-		}
-
-		/* we do not need strong random numbers here.
-		 */
-		ret = _gnutls_rnd(GNUTLS_RND_NONCE, session->key.key.data,
-				  session->key.key.size);
-		if (ret < 0) {
-			gnutls_assert();
-			return ret;
-		}
-
+	if (use_rnd_key != 0) {
+		session->key.key.data = rndkey.data;
+		session->key.key.size = rndkey.size;
+		rndkey.data = NULL;
 	} else {
 		session->key.key.data = plaintext.data;
 		session->key.key.size = plaintext.size;
@@ -201,7 +227,10 @@ proc_rsa_client_kx(gnutls_session_t session, uint8_t * data,
 	session->key.key.data[0] = _gnutls_get_adv_version_major(session);
 	session->key.key.data[1] = _gnutls_get_adv_version_minor(session);
 
-	return 0;
+	ret = 0;
+ cleanup:
+	gnutls_free(rndkey.data);
+	return ret;
 }
 
 
