@@ -20,10 +20,10 @@
  *
  */
 
-#include <gnutls_int.h>
-#include <gnutls_errors.h>
-#include <gnutls_cipher_int.h>
-#include <gnutls_datum.h>
+#include "gnutls_int.h"
+#include "errors.h"
+#include <cipher_int.h>
+#include <datum.h>
 #include <gnutls/crypto.h>
 #include <algorithms.h>
 #include <random.h>
@@ -734,8 +734,7 @@ gnutls_aead_cipher_decrypt(gnutls_aead_cipher_hd_t handle,
  *
  * This function will encrypt the given data using the algorithm
  * specified by the context. The output data will contain the
- * authentication tag. This function requires that 
- * gnutls_aead_cipher_set_nonce() is called before it.
+ * authentication tag.
  *
  * Returns: Zero or a negative error code on error.
  *
