@@ -19,15 +19,23 @@
  */
 
 #include "gnutls_int.h"
-#include "gnutls_auth.h"
-#include "gnutls_errors.h"
-#include "gnutls_num.h"
+#include "auth.h"
+#include "errors.h"
+#include "num.h"
 #include <ext/dumbfw.h>
 
+/* This extension adds additional padding data in the TLS client hello.
+ * There is an issue with some firewalls [0] rejecting TLS client hello
+ * data that are between 256 and 511 bytes, and this extension will
+ * make sure that client hello isn't in this range.
+ *
+ * [0]. http://www.ietf.org/mail-archive/web/tls/current/msg10423.html
+ */
+
 static int _gnutls_dumbfw_send_params(gnutls_session_t session,
 				    gnutls_buffer_st * extdata);
 
-extension_entry_st ext_mod_dumbfw = {
+const extension_entry_st ext_mod_dumbfw = {
 	.name = "DUMBFW",
 	.type = GNUTLS_EXTENSION_DUMBFW,
 	.parse_type = GNUTLS_EXT_APPLICATION,
@@ -44,22 +52,26 @@ _gnutls_dumbfw_send_params(gnutls_session_t session,
 			 gnutls_buffer_st * extdata)
 {
 	int total_size = 0, ret;
-	uint8_t pad[DUMBFW_PADDING_SIZE];
+	uint8_t pad[257];
+	unsigned pad_size;
 
 	if (session->security_parameters.entity == GNUTLS_SERVER ||
-	    session->internals.priorities.dumbfw == 0) {
+	    session->internals.priorities.dumbfw == 0 ||
+	    IS_DTLS(session) != 0 ||
+	    (extdata->length < 256 || extdata->length >= 512)) {
 		return 0;
 	} else {
-	        memset(pad, 0, sizeof(pad));
+		/* 256 <= extdata->length < 512 */
+		pad_size = 512 - extdata->length;
+	        memset(pad, 0, pad_size);
 
 		ret =
-		    _gnutls_buffer_append_data_prefix(extdata, 16,
-							      pad,
-							      sizeof(pad)-2);
+		    gnutls_buffer_append_data(extdata, pad,
+						       pad_size);
 		if (ret < 0)
 			return gnutls_assert_val(ret);
 
-		total_size += sizeof(pad);
+		total_size += pad_size;
 	}
 
 	return total_size;
