@@ -24,9 +24,9 @@
  */
 
 #include "gnutls_int.h"
-#include "gnutls_errors.h"
-#include "gnutls_num.h"
-#include <gnutls_extensions.h>
+#include "errors.h"
+#include "num.h"
+#include <extensions.h>
 #include <ext/max_record.h>
 
 static int _gnutls_max_record_recv_params(gnutls_session_t session,
@@ -47,7 +47,7 @@ static int _gnutls_mre_num2record(int num);
 static int _gnutls_mre_record2num(uint16_t record_size);
 
 
-extension_entry_st ext_mod_max_record_size = {
+const extension_entry_st ext_mod_max_record_size = {
 	.name = "MAX RECORD SIZE",
 	.type = GNUTLS_EXTENSION_MAX_RECORD_SIZE,
 	.parse_type = GNUTLS_EXT_TLS,
