@@ -20,9 +20,9 @@
  *
  */
 
-#include <gnutls_int.h>
+#include "gnutls_int.h"
 #include <ext/safe_renegotiation.h>
-#include <gnutls_errors.h>
+#include "errors.h"
 
 
 static int _gnutls_sr_recv_params(gnutls_session_t state,
@@ -31,7 +31,7 @@ static int _gnutls_sr_send_params(gnutls_session_t state,
 				  gnutls_buffer_st *);
 static void _gnutls_sr_deinit_data(extension_priv_data_t priv);
 
-extension_entry_st ext_mod_sr = {
+const extension_entry_st ext_mod_sr = {
 	.name = "SAFE RENEGOTIATION",
 	.type = GNUTLS_EXTENSION_SAFE_RENEGOTIATION,
 	.parse_type = GNUTLS_EXT_MANDATORY,
@@ -60,6 +60,10 @@ _gnutls_ext_sr_finished(gnutls_session_t session, void *vdata,
 					   &epriv);
 	if (ret < 0) {
 		gnutls_assert();
+		/* if a client didn't advertise safe renegotiation, we treat
+		 * it as disabled. */
+		if (session->security_parameters.entity == GNUTLS_SERVER)
+			return 0;
 		return ret;
 	}
 	priv = epriv;
@@ -215,6 +219,7 @@ int _gnutls_ext_sr_recv_cs(gnutls_session_t session)
 
 	priv->safe_renegotiation_received = 1;
 	priv->connection_using_safe_renegotiation = 1;
+	_gnutls_extension_list_add(session, GNUTLS_EXTENSION_SAFE_RENEGOTIATION);
 
 	if (set != 0)
 		_gnutls_ext_set_session_data(session,
