@@ -21,9 +21,9 @@
  */
 
 #include "gnutls_int.h"
-#include "gnutls_auth.h"
-#include "gnutls_errors.h"
-#include "gnutls_num.h"
+#include "auth.h"
+#include "errors.h"
+#include "num.h"
 #include <ext/server_name.h>
 #ifdef HAVE_LIBIDN
 # include <idna.h>
@@ -43,7 +43,7 @@ static int _gnutls_server_name_pack(extension_priv_data_t _priv,
 static void _gnutls_server_name_deinit_data(extension_priv_data_t priv);
 
 
-extension_entry_st ext_mod_server_name = {
+const extension_entry_st ext_mod_server_name = {
 	.name = "SERVER NAME",
 	.type = GNUTLS_EXTENSION_SERVER_NAME,
 	.parse_type = GNUTLS_EXT_APPLICATION,
@@ -68,7 +68,7 @@ static int
 _gnutls_server_name_recv_params(gnutls_session_t session,
 				const uint8_t * data, size_t _data_size)
 {
-	int i, j;
+	int i;
 	const unsigned char *p;
 	uint16_t len, type;
 	ssize_t data_size = _data_size;
@@ -128,8 +128,10 @@ _gnutls_server_name_recv_params(gnutls_session_t session,
 			return GNUTLS_E_MEMORY_ERROR;
 		}
 
+		priv->server_names_size = server_names;
+
 		p = data + 2;
-		for (j = i = 0; i < server_names; i++) {
+		for (i = 0; i < server_names; i++) {
 			type = *p;
 			p++;
 
@@ -139,17 +141,13 @@ _gnutls_server_name_recv_params(gnutls_session_t session,
 			switch (type) {
 			case 0:	/* NAME_DNS */
 				if (len < MAX_SERVER_NAME_SIZE) {
-					memcpy(priv->server_names[j].name,
+					memcpy(priv->server_names[i].name,
 					       p, len);
-					priv->server_names[j].name[len] = 0;
-					priv->server_names[j].name_length =
-					    strlen((char*)priv->server_names[j].name);
-					if (priv->server_names[j].name_length == len) {
-						/* valid ascii with no embedded NULL */
-						priv->server_names[j].type =
-						    GNUTLS_NAME_DNS;
-						j++;
-					}
+					priv->server_names[i].name[len] = 0;
+					priv->server_names[i].name_length =
+					    len;
+					priv->server_names[i].type =
+					    GNUTLS_NAME_DNS;
 					break;
 				}
 			}
@@ -158,8 +156,6 @@ _gnutls_server_name_recv_params(gnutls_session_t session,
 			p += len;
 		}
 
-		priv->server_names_size = j;
-
 		epriv = priv;
 		_gnutls_ext_set_session_data(session,
 					     GNUTLS_EXTENSION_SERVER_NAME,
@@ -189,6 +185,7 @@ _gnutls_server_name_send_params(gnutls_session_t session,
 	if (ret < 0)
 		return 0;
 
+
 	/* this function sends the client extension data (dnsname)
 	 */
 	if (session->security_parameters.entity == GNUTLS_CLIENT) {
