@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009-2012 Free Software Foundation, Inc.
+ * Copyright (C) 2009-2016 Free Software Foundation, Inc.
  *
  * Author: Daiki Ueno
  *
@@ -20,28 +20,33 @@
  *
  */
 
-#include <gnutls_int.h>
-#include <gnutls_errors.h>
-#include <gnutls_datum.h>
+#include "gnutls_int.h"
+#include "errors.h"
+#include <datum.h>
 #include <algorithms.h>
-#include <gnutls_handshake.h>
-#include <gnutls_num.h>
-#include <gnutls_constate.h>
-#include <gnutls_session_pack.h>
+#include <handshake.h>
+#include <num.h>
+#include <constate.h>
+#include <session_pack.h>
 #include <random.h>
 #include <ext/session_ticket.h>
-#include <gnutls_mbuffers.h>
-#include <gnutls_extensions.h>
-#include <gnutls_constate.h>
-#include <gnutls_dtls.h>
+#include <mbuffers.h>
+#include <extensions.h>
+#include <constate.h>
+#include <dtls.h>
+#include "db.h"
 
 #ifdef ENABLE_SESSION_TICKETS
 
-#define KEY_NAME_SIZE SESSION_TICKET_KEY_NAME_SIZE
-#define KEY_SIZE SESSION_TICKET_KEY_SIZE
-#define IV_SIZE 12 /* GCM */
+#define KEY_NAME_SIZE 16
+#define CIPHER_KEY_SIZE 32
+#define CIPHER GNUTLS_CIPHER_AES_256_CBC
+#define IV_SIZE 16
+#define BLOCK_SIZE 16
 
-#define TAG_SIZE 16 /* GCM */
+#define MAC_SECRET_SIZE 16
+#define MAC_ALGO GNUTLS_MAC_SHA1
+#define MAC_SIZE 20 /* HMAC-SHA1 */
 
 static int session_ticket_recv_params(gnutls_session_t session,
 				      const uint8_t * data,
@@ -54,10 +59,10 @@ static int session_ticket_pack(extension_priv_data_t _priv,
 			       gnutls_buffer_st * ps);
 static void session_ticket_deinit_data(extension_priv_data_t priv);
 
-extension_entry_st ext_mod_session_ticket = {
+const extension_entry_st ext_mod_session_ticket = {
 	.name = "SESSION TICKET",
 	.type = GNUTLS_EXTENSION_SESSION_TICKET,
-	.parse_type = GNUTLS_EXT_MANDATORY,
+	.parse_type = GNUTLS_EXT_TLS,
 
 	.recv_func = session_ticket_recv_params,
 	.send_func = session_ticket_send_params,
@@ -66,9 +71,10 @@ extension_entry_st ext_mod_session_ticket = {
 	.deinit_func = session_ticket_deinit_data,
 };
 
-#define SESSION_KEY_SIZE (SESSION_TICKET_KEY_NAME_SIZE+SESSION_TICKET_KEY_SIZE)
+#define SESSION_KEY_SIZE (KEY_NAME_SIZE+CIPHER_KEY_SIZE+MAC_SECRET_SIZE)
 #define NAME_POS (0)
-#define KEY_POS (SESSION_TICKET_KEY_NAME_SIZE)
+#define KEY_POS (KEY_NAME_SIZE)
+#define MAC_SECRET_POS (KEY_NAME_SIZE+CIPHER_KEY_SIZE)
 
 typedef struct {
 	int session_ticket_enable;
@@ -85,27 +91,66 @@ struct ticket_st {
 	uint8_t IV[IV_SIZE];
 	uint8_t *encrypted_state;
 	uint16_t encrypted_state_len;
-	uint8_t tag[TAG_SIZE];
+	uint8_t mac[MAC_SIZE];
 };
 
+static
+int digest_ticket(const gnutls_datum_t * key, struct ticket_st *ticket,
+	      uint8_t * digest)
+{
+	mac_hd_st digest_hd;
+	uint16_t length16;
+	int ret;
+
+	ret = _gnutls_mac_init(&digest_hd, mac_to_entry(MAC_ALGO),
+			      key->data, key->size);
+	if (ret < 0) {
+		gnutls_assert();
+		return ret;
+	}
+
+	_gnutls_mac(&digest_hd, ticket->key_name, KEY_NAME_SIZE);
+	_gnutls_mac(&digest_hd, ticket->IV, IV_SIZE);
+	length16 = _gnutls_conv_uint16(ticket->encrypted_state_len);
+	_gnutls_mac(&digest_hd, &length16, 2);
+	_gnutls_mac(&digest_hd, ticket->encrypted_state,
+		   ticket->encrypted_state_len);
+	_gnutls_mac_deinit(&digest_hd, digest);
+
+	return 0;
+}
+
 static int
 decrypt_ticket(gnutls_session_t session, session_ticket_ext_st * priv,
 	       struct ticket_st *ticket)
 {
 	cipher_hd_st cipher_hd;
-	gnutls_datum_t key, IV, state;
-	uint8_t final[TAG_SIZE];
+	gnutls_datum_t key, IV, state, mac_secret;
+	uint8_t cmac[MAC_SIZE];
 	time_t timestamp = gnutls_time(0);
 	int ret;
 
-	/* Decrypt encrypted_state using 128-bit AES in CBC mode. */
+	/* Check the integrity of ticket */
+	mac_secret.data = (void *) &priv->key[MAC_SECRET_POS];
+	mac_secret.size = MAC_SECRET_SIZE;
+	ret = digest_ticket(&mac_secret, ticket, cmac);
+	if (ret < 0)
+		return gnutls_assert_val(ret);
+
+	if (memcmp(ticket->mac, cmac, MAC_SIZE))
+		return gnutls_assert_val(GNUTLS_E_DECRYPTION_FAILED);
+
+	if (ticket->encrypted_state_len % BLOCK_SIZE != 0)
+		return gnutls_assert_val(GNUTLS_E_DECRYPTION_FAILED);
+
+	/* Decrypt encrypted_state */
 	key.data = (void *) &priv->key[KEY_POS];
-	key.size = KEY_SIZE;
+	key.size = CIPHER_KEY_SIZE;
 	IV.data = ticket->IV;
 	IV.size = IV_SIZE;
 	ret =
 	    _gnutls_cipher_init(&cipher_hd,
-				cipher_to_entry(GNUTLS_CIPHER_AES_128_GCM),
+				cipher_to_entry(CIPHER),
 				&key, &IV, 0);
 	if (ret < 0) {
 		gnutls_assert();
@@ -117,13 +162,6 @@ decrypt_ticket(gnutls_session_t session, session_ticket_ext_st * priv,
 		gnutls_assert();
 		goto cleanup;
 	}
-	
-	_gnutls_cipher_tag(&cipher_hd, final, TAG_SIZE);
-	if (memcmp(ticket->tag, final, TAG_SIZE) != 0) {
-		gnutls_assert();
-		ret = GNUTLS_E_DECRYPTION_FAILED;
-		goto cleanup;
-	}
 
 	/* Unpack security parameters. */
 	state.data = ticket->encrypted_state;
@@ -144,13 +182,19 @@ decrypt_ticket(gnutls_session_t session, session_ticket_ext_st * priv,
 		goto cleanup;
 	}
 
+	ret = _gnutls_check_resumed_params(session);
+	if (ret < 0) {
+		gnutls_assert();
+		goto cleanup;
+	}
+
 	session->internals.resumed = RESUME_TRUE;
 
 	ret = 0;
 cleanup:
-        _gnutls_cipher_deinit(&cipher_hd);
-        
-        return ret;
+	_gnutls_cipher_deinit(&cipher_hd);
+
+	return ret;
 
 }
 
@@ -162,6 +206,7 @@ encrypt_ticket(gnutls_session_t session, session_ticket_ext_st * priv,
 	gnutls_datum_t key, IV;
 	gnutls_datum_t state = {NULL,0}, encrypted_state = {NULL,0};
 	uint8_t iv[IV_SIZE];
+	gnutls_datum_t mac_secret;
 	uint32_t t;
 	int ret;
 
@@ -172,22 +217,21 @@ encrypt_ticket(gnutls_session_t session, session_ticket_ext_st * priv,
 		return ret;
 	}
 
-	encrypted_state.size = state.size;
-	encrypted_state.data = gnutls_malloc(encrypted_state.size);
+	encrypted_state.size = ((state.size + BLOCK_SIZE - 1) / BLOCK_SIZE) * BLOCK_SIZE;
+	encrypted_state.data = gnutls_calloc(1, encrypted_state.size);
 	if (!encrypted_state.data) {
 		gnutls_assert();
 		ret = GNUTLS_E_MEMORY_ERROR;
 		goto cleanup;
 	}
-	memset(encrypted_state.data, 0, encrypted_state.size);
 	memcpy(encrypted_state.data, state.data, state.size);
 
-	/* Encrypt state using 128-bit AES in GCM mode. */
+	/* Encrypt state */
 	key.data = (void *) &priv->key[KEY_POS];
-	key.size = KEY_SIZE;
+	key.size = CIPHER_KEY_SIZE;
 	IV.data = iv;
 	IV.size = IV_SIZE;
-	
+
 	t = gnutls_time(0);
 	memcpy(iv, &t, 4);
 	ret = _gnutls_rnd(GNUTLS_RND_NONCE, iv+4, IV_SIZE-4);
@@ -198,7 +242,7 @@ encrypt_ticket(gnutls_session_t session, session_ticket_ext_st * priv,
 
 	ret =
 	    _gnutls_cipher_init(&cipher_hd,
-				cipher_to_entry(GNUTLS_CIPHER_AES_128_GCM),
+				cipher_to_entry(CIPHER),
 				&key, &IV, 1);
 	if (ret < 0) {
 		gnutls_assert();
@@ -211,15 +255,22 @@ encrypt_ticket(gnutls_session_t session, session_ticket_ext_st * priv,
 		gnutls_assert();
 		goto cleanup2;
 	}
-	
-	_gnutls_cipher_tag(&cipher_hd, ticket->tag, TAG_SIZE);
+
 
 	/* Fill the ticket structure to compute MAC. */
 	memcpy(ticket->key_name, &priv->key[NAME_POS], KEY_NAME_SIZE);
 	memcpy(ticket->IV, IV.data, IV.size);
 	ticket->encrypted_state_len = encrypted_state.size;
 	ticket->encrypted_state = encrypted_state.data;
-	
+
+	mac_secret.data = &priv->key[MAC_SECRET_POS];
+	mac_secret.size = MAC_SECRET_SIZE;
+	ret = digest_ticket(&mac_secret, ticket, ticket->mac);
+	if (ret < 0) {
+		gnutls_assert();
+		goto cleanup2;
+	}
+
 	encrypted_state.data = NULL;
 
 	ret = 0;
@@ -250,7 +301,7 @@ session_ticket_recv_params(gnutls_session_t session,
 	if (ret < 0) {
 		return 0;
 	}
-	priv = epriv.ptr;
+	priv = epriv;
 
 	if (!priv->session_ticket_enable)
 		return 0;
@@ -258,7 +309,6 @@ session_ticket_recv_params(gnutls_session_t session,
 	if (session->security_parameters.entity == GNUTLS_SERVER) {
 		struct ticket_st ticket;
 		const uint8_t *encrypted_state;
-		int ret;
 
 		/* The client requested a new session ticket. */
 		if (data_size == 0) {
@@ -266,6 +316,13 @@ session_ticket_recv_params(gnutls_session_t session,
 			return 0;
 		}
 
+		/* Format:
+		 *  Key name
+		 *  IV
+		 *  data length
+		 *  encrypted data
+		 *  MAC
+		 */
 		DECR_LEN(data_size, KEY_NAME_SIZE);
 		memcpy(ticket.key_name, data, KEY_NAME_SIZE);
 		data += KEY_NAME_SIZE;
@@ -292,8 +349,8 @@ session_ticket_recv_params(gnutls_session_t session,
 		DECR_LEN(data_size, ticket.encrypted_state_len);
 		data += ticket.encrypted_state_len;
 
-		DECR_LEN(data_size, TAG_SIZE);
-		memcpy(ticket.tag, data, TAG_SIZE);
+		DECR_LEN(data_size, MAC_SIZE);
+		memcpy(ticket.mac, data, MAC_SIZE);
 
 		ticket.encrypted_state =
 		    gnutls_malloc(ticket.encrypted_state_len);
@@ -305,7 +362,10 @@ session_ticket_recv_params(gnutls_session_t session,
 		       ticket.encrypted_state_len);
 
 		ret = decrypt_ticket(session, priv, &ticket);
+
 		gnutls_free(ticket.encrypted_state);
+		ticket.encrypted_state = NULL;
+
 		if (ret < 0) {
 			priv->session_ticket_renew = 1;
 			return 0;
@@ -337,7 +397,7 @@ session_ticket_send_params(gnutls_session_t session,
 					 GNUTLS_EXTENSION_SESSION_TICKET,
 					 &epriv);
 	if (ret >= 0)
-		priv = epriv.ptr;
+		priv = epriv;
 
 	if (priv == NULL || !priv->session_ticket_enable)
 		return 0;
@@ -352,7 +412,7 @@ session_ticket_send_params(gnutls_session_t session,
 							 GNUTLS_EXTENSION_SESSION_TICKET,
 							 &epriv);
 		if (ret >= 0)
-			priv = epriv.ptr;
+			priv = epriv;
 
 		/* no previous data. Just advertize it */
 		if (ret < 0)
@@ -381,7 +441,7 @@ session_ticket_send_params(gnutls_session_t session,
 
 static void session_ticket_deinit_data(extension_priv_data_t epriv)
 {
-	session_ticket_ext_st *priv = epriv.ptr;
+	session_ticket_ext_st *priv = epriv;
 
 	gnutls_free(priv->session_ticket);
 	gnutls_free(priv);
@@ -390,7 +450,7 @@ static void session_ticket_deinit_data(extension_priv_data_t epriv)
 static int
 session_ticket_pack(extension_priv_data_t epriv, gnutls_buffer_st * ps)
 {
-	session_ticket_ext_st *priv = epriv.ptr;
+	session_ticket_ext_st *priv = epriv;
 	int ret;
 
 	BUFFER_APPEND_PFX4(ps, priv->session_ticket,
@@ -419,7 +479,7 @@ session_ticket_unpack(gnutls_buffer_st * ps, extension_priv_data_t * _priv)
 	priv->session_ticket_len = ticket.size;
 	BUFFER_POP_NUM(ps, priv->session_ticket_enable);
 
-	epriv.ptr = priv;
+	epriv = priv;
 	*_priv = epriv;
 
 	return 0;
@@ -451,7 +511,7 @@ int gnutls_session_ticket_key_generate(gnutls_datum_t * key)
 
 /**
  * gnutls_session_ticket_enable_client:
- * @session: is a #gnutls_session_t structure.
+ * @session: is a #gnutls_session_t type.
  *
  * Request that the client should attempt session resumption using
  * SessionTicket.
@@ -477,7 +537,7 @@ int gnutls_session_ticket_enable_client(gnutls_session_t session)
 		return GNUTLS_E_MEMORY_ERROR;
 	}
 	priv->session_ticket_enable = 1;
-	epriv.ptr = priv;
+	epriv = priv;
 
 	_gnutls_ext_set_session_data(session,
 				     GNUTLS_EXTENSION_SESSION_TICKET,
@@ -488,12 +548,13 @@ int gnutls_session_ticket_enable_client(gnutls_session_t session)
 
 /**
  * gnutls_session_ticket_enable_server:
- * @session: is a #gnutls_session_t structure.
+ * @session: is a #gnutls_session_t type.
  * @key: key to encrypt session parameters.
  *
  * Request that the server should attempt session resumption using
  * SessionTicket.  @key must be initialized with
- * gnutls_session_ticket_key_generate().
+ * gnutls_session_ticket_key_generate(), and should be overwritten
+ * using gnutls_memset() before being released.
  *
  * Returns: On success, %GNUTLS_E_SUCCESS (0) is returned, or an
  * error code.
@@ -517,7 +578,7 @@ gnutls_session_ticket_enable_server(gnutls_session_t session,
 		gnutls_assert();
 		return GNUTLS_E_MEMORY_ERROR;
 	}
-	epriv.ptr = priv;
+	epriv = priv;
 
 	memcpy(&priv->key, key->data, key->size);
 	priv->session_ticket_enable = 1;
@@ -548,7 +609,7 @@ int _gnutls_send_new_session_ticket(gnutls_session_t session, int again)
 						 &epriv);
 		if (ret < 0)
 			return 0;
-		priv = epriv.ptr;
+		priv = epriv;
 
 		if (!priv->session_ticket_renew)
 			return 0;
@@ -578,7 +639,7 @@ int _gnutls_send_new_session_ticket(gnutls_session_t session, int again)
 
 		ticket_len =
 		    KEY_NAME_SIZE + IV_SIZE + 2 +
-		    ticket.encrypted_state_len + TAG_SIZE;
+		    ticket.encrypted_state_len + MAC_SIZE;
 
 		bufel =
 		    _gnutls_handshake_alloc(session, 
@@ -611,8 +672,8 @@ int _gnutls_send_new_session_ticket(gnutls_session_t session, int again)
 		gnutls_free(ticket.encrypted_state);
 		p += ticket.encrypted_state_len;
 
-		memcpy(p, ticket.tag, TAG_SIZE);
-		p += TAG_SIZE;
+		memcpy(p, ticket.mac, MAC_SIZE);
+		p += MAC_SIZE;
 
 		data_size = p - data;
 
@@ -640,7 +701,7 @@ int _gnutls_recv_new_session_ticket(gnutls_session_t session)
 		gnutls_assert();
 		return 0;
 	}
-	priv = epriv.ptr;
+	priv = epriv;
 
 	if (!priv->session_ticket_renew)
 		return 0;
