@@ -19,14 +19,15 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>
  *
  */
-#include <gnutls_int.h>
+#include "gnutls_int.h"
 #include <gnutls/gnutls.h>
 #include <gnutls/crypto.h>
 #include <unistd.h>
-#include <gnutls_errors.h>
+#include "errors.h"
 #include <fips.h>
 #include <gnutls/self-test.h>
 #include <stdio.h>
+#include <extras/hex.h>
 #include <random.h>
 
 unsigned int _gnutls_lib_mode = LIB_STATE_POWERON;
@@ -54,12 +55,12 @@ const char *p;
 	if (_fips_mode != -1)
 		return _fips_mode;
 
-	p = secure_getenv("GNUTLS_SKIP_FIPS_INTEGRITY_CHECKS");
+	p = getenv("GNUTLS_SKIP_FIPS_INTEGRITY_CHECKS");
 	if (p && p[0] == '1') {
 		_skip_integrity_checks = 1;
 	}
 
-	p = secure_getenv("GNUTLS_FORCE_FIPS_MODE");
+	p = getenv("GNUTLS_FORCE_FIPS_MODE");
 	if (p) {
 		if (p[0] == '1')
 			_fips_mode = 1;
@@ -113,8 +114,6 @@ void _gnutls_fips_mode_reset_zombie(void)
 #define HOGWEED_LIBRARY_NAME "libhogweed.so.2"
 #define GMP_LIBRARY_NAME "libgmp.so.10"
 
-static const char fips_key[] = "orboDeJITITejsirpADONivirpUkvarP";
-
 #define HMAC_SUFFIX ".hmac"
 #define HMAC_SIZE 32
 #define HMAC_ALGO GNUTLS_MAC_SHA256
@@ -202,7 +201,7 @@ static unsigned check_binary_integrity(const char* libname, const char* symbol)
 
 	prev = _gnutls_get_lib_state();
 	_gnutls_switch_lib_state(LIB_STATE_OPERATIONAL);
-	ret = gnutls_hmac_fast(HMAC_ALGO, fips_key, sizeof(fips_key)-1,
+	ret = gnutls_hmac_fast(HMAC_ALGO, FIPS_KEY, sizeof(FIPS_KEY)-1,
 		data.data, data.size, new_hmac);
 	_gnutls_switch_lib_state(prev);
 	
@@ -224,8 +223,8 @@ static unsigned check_binary_integrity(const char* libname, const char* symbol)
 		}
 	}
 
-	hmac_size = sizeof(hmac);
-	ret = _gnutls_hex2bin((void*)data.data, data.size, hmac, &hmac_size);
+	hmac_size = hex_data_size(data.size);
+	ret = gnutls_hex_decode(&data, hmac, &hmac_size);
 	gnutls_free(data.data);
 
 	if (ret < 0) {
