@@ -1,5 +1,6 @@
 /*
- * Copyright (C) 2003-2012 Free Software Foundation, Inc.
+ * Copyright (C) 2003-2016 Free Software Foundation, Inc.
+ * Copyright (C) 2015-2016 Red Hat, Inc.
  *
  * Author: Nikos Mavrogiannopoulos
  *
@@ -86,6 +87,8 @@ extern "C" {
 
 #define GNUTLS_FSAN_SET 0
 #define GNUTLS_FSAN_APPEND 1
+#define GNUTLS_FSAN_ENCODE_OCTET_STRING (1<<1)
+#define GNUTLS_FSAN_ENCODE_UTF8_STRING (1<<2)
 
 #define GNUTLS_X509EXT_OID_SUBJECT_KEY_ID "2.5.29.14"
 #define GNUTLS_X509EXT_OID_KEY_USAGE "2.5.29.15"
@@ -124,6 +127,10 @@ typedef enum gnutls_certificate_import_flags {
 
 int gnutls_x509_crt_init(gnutls_x509_crt_t * cert);
 void gnutls_x509_crt_deinit(gnutls_x509_crt_t cert);
+
+unsigned gnutls_x509_crt_equals(gnutls_x509_crt_t cert1, gnutls_x509_crt_t cert2);
+unsigned gnutls_x509_crt_equals2(gnutls_x509_crt_t cert1, gnutls_datum_t * der);
+
 int gnutls_x509_crt_import(gnutls_x509_crt_t cert,
 			   const gnutls_datum_t * data,
 			   gnutls_x509_crt_fmt_t format);
@@ -190,6 +197,8 @@ int gnutls_x509_crt_get_signature(gnutls_x509_crt_t cert,
 				  char *sig, size_t * sizeof_sig);
 int gnutls_x509_crt_get_version(gnutls_x509_crt_t cert);
 
+int gnutls_x509_crt_get_pk_oid(gnutls_x509_crt_t cert, char *oid, size_t *oid_size);
+int gnutls_x509_crt_get_signature_oid(gnutls_x509_crt_t cert, char *oid, size_t *oid_size);
 
 /**
  * gnutls_keyid_flags_t:
@@ -563,12 +572,26 @@ int gnutls_x509_crt_set_subject_alt_name(gnutls_x509_crt_t crt,
 					 unsigned int data_size,
 					 unsigned int flags);
 
+int
+gnutls_x509_crt_set_subject_alt_othername(gnutls_x509_crt_t crt,
+				     const char *oid,
+				     const void *data,
+				     unsigned int data_size,
+				     unsigned int flags);
+
 int gnutls_x509_crt_set_issuer_alt_name(gnutls_x509_crt_t crt,
 					 gnutls_x509_subject_alt_name_t
 					 type, const void *data,
 					 unsigned int data_size,
 					 unsigned int flags);
 
+int
+gnutls_x509_crt_set_issuer_alt_othername(gnutls_x509_crt_t crt,
+				     const char *oid,
+				     const void *data,
+				     unsigned int data_size,
+				     unsigned int flags);
+
 int gnutls_x509_crt_sign(gnutls_x509_crt_t crt,
 			 gnutls_x509_crt_t issuer,
 			 gnutls_x509_privkey_t issuer_key);
@@ -690,6 +713,8 @@ int gnutls_x509_crl_get_signature(gnutls_x509_crl_t crl,
 				  char *sig, size_t * sizeof_sig);
 int gnutls_x509_crl_get_version(gnutls_x509_crl_t crl);
 
+int gnutls_x509_crl_get_signature_oid(gnutls_x509_crl_t crl, char *oid, size_t *oid_size);
+
 time_t gnutls_x509_crl_get_this_update(gnutls_x509_crl_t crl);
 time_t gnutls_x509_crl_get_next_update(gnutls_x509_crl_t crl);
 
@@ -1063,6 +1088,35 @@ int gnutls_x509_privkey_get_key_id(gnutls_x509_privkey_t key,
 int gnutls_x509_privkey_generate(gnutls_x509_privkey_t key,
 				 gnutls_pk_algorithm_t algo,
 				 unsigned int bits, unsigned int flags);
+
+void gnutls_x509_privkey_set_flags(gnutls_x509_privkey_t key, unsigned int flags);
+
+/**
+ * gnutls_keygen_types_t:
+ * @GNUTLS_KEYGEN_SEED: Specifies the seed to be used in key generation.
+ * @GNUTLS_KEYGEN_DIGEST: The size field specifies the hash algorithm to be used in key generation.
+ *
+ * Enumeration of different key exchange algorithms.
+ */
+typedef enum {
+	GNUTLS_KEYGEN_SEED = 1,
+	GNUTLS_KEYGEN_DIGEST = 2,
+} gnutls_keygen_types_t;
+
+typedef struct {
+	gnutls_keygen_types_t type;
+	unsigned char *data;
+	unsigned int size;
+} gnutls_keygen_data_st;
+
+int
+gnutls_x509_privkey_generate2(gnutls_x509_privkey_t key,
+			      gnutls_pk_algorithm_t algo, unsigned int bits,
+			      unsigned int flags, const gnutls_keygen_data_st *data, unsigned data_size);
+
+int gnutls_x509_privkey_verify_seed(gnutls_x509_privkey_t key, gnutls_digest_algorithm_t, const void *seed, size_t seed_size);
+int gnutls_x509_privkey_get_seed(gnutls_x509_privkey_t key, gnutls_digest_algorithm_t*, void *seed, size_t *seed_size);
+
 int gnutls_x509_privkey_verify_params(gnutls_x509_privkey_t key);
 
 int gnutls_x509_privkey_export(gnutls_x509_privkey_t key,
@@ -1199,6 +1253,13 @@ int gnutls_x509_crq_set_subject_alt_name(gnutls_x509_crq_t crq,
 					 unsigned int data_size,
 					 unsigned int flags);
 
+int
+gnutls_x509_crq_set_subject_alt_othername(gnutls_x509_crq_t crq,
+				     const char *oid,
+				     const void *data,
+				     unsigned int data_size,
+				     unsigned int flags);
+
 int gnutls_x509_crq_set_key_usage(gnutls_x509_crq_t crq,
 				  unsigned int usage);
 int gnutls_x509_crq_set_basic_constraints(gnutls_x509_crq_t crq,
@@ -1232,6 +1293,9 @@ int gnutls_x509_crq_get_attribute_info(gnutls_x509_crq_t crq,
 int gnutls_x509_crq_get_pk_algorithm(gnutls_x509_crq_t crq,
 				     unsigned int *bits);
 
+int gnutls_x509_crq_get_signature_oid(gnutls_x509_crq_t crq, char *oid, size_t *oid_size);
+int gnutls_x509_crq_get_pk_oid(gnutls_x509_crq_t crq, char *oid, size_t *oid_size);
+
 int gnutls_x509_crq_get_key_id(gnutls_x509_crq_t crq,
 			       unsigned int flags,
 			       unsigned char *output_data,
