@@ -18,15 +18,15 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>
  */
 
-#include <gnutls_int.h>
+#include "gnutls_int.h"
 #include <gnutls/pkcs11.h>
 #include <stdio.h>
 #include <string.h>
-#include <gnutls_errors.h>
-#include <gnutls_datum.h>
+#include "errors.h"
+#include <datum.h>
 #include <pkcs11_int.h>
-#include <gnutls_sig.h>
-#include <gnutls_pk.h>
+#include <tls-sig.h>
+#include <pk.h>
 #include <fips.h>
 #include "urls.h"
 #include <p11-kit/uri.h>
@@ -655,6 +655,8 @@ gnutls_pkcs11_privkey_generate2(const char *url, gnutls_pk_algorithm_t pk,
 }
 #endif
 
+static const char def_rsa_pub_exp[3] = { 1,0,1 }; // 65537 = 0x10001
+
 struct dsa_params {
 	/* FIPS 186-3 maximal size for L and N length pair is (3072,256). */
 	uint8_t prime[384];
@@ -758,7 +760,6 @@ gnutls_pkcs11_privkey_generate3(const char *url, gnutls_pk_algorithm_t pk,
 	gnutls_pkcs11_obj_t obj = NULL;
 	gnutls_datum_t der = {NULL, 0};
 	ck_key_type_t key_type;
-	char pubEx[3] = { 1,0,1 }; // 65537 = 0x10001
 	uint8_t id[20];
 	struct dsa_params dsa_params;
 
@@ -862,8 +863,8 @@ gnutls_pkcs11_privkey_generate3(const char *url, gnutls_pk_algorithm_t pk,
 		a_val++;
 
 		a[a_val].type = CKA_PUBLIC_EXPONENT;
-		a[a_val].value = pubEx;
-		a[a_val].value_len = sizeof(pubEx);
+		a[a_val].value = (char*)def_rsa_pub_exp;
+		a[a_val].value_len = sizeof(def_rsa_pub_exp);
 		a_val++;
 
 		break;
