@@ -22,8 +22,8 @@
 
 #include <config.h>
 #include <system.h>
-#include <gnutls_int.h>
-#include <gnutls_errors.h>
+#include "gnutls_int.h"
+#include "errors.h"
 
 #include <sys/socket.h>
 #include <errno.h>
@@ -45,8 +45,9 @@ static HMODULE Crypt32_dll;
 #  define pCertEnumCRLsInStore CertEnumCRLsInStore
 # endif
 
-#else /* _WIN32 */
-# include <sys/select.h>
+#else /* !_WIN32 */
+
+# include <poll.h>
 
 # ifdef HAVE_PTHREAD_LOCKS
 #  include <pthread.h>
@@ -129,8 +130,13 @@ ssize_t
 system_writev(gnutls_transport_ptr_t ptr, const giovec_t * iovec,
 	      int iovec_cnt)
 {
-	return writev(GNUTLS_POINTER_TO_INT(ptr), (struct iovec *) iovec,
-		      iovec_cnt);
+	struct msghdr hdr;
+
+	memset(&hdr, 0, sizeof(hdr));
+	hdr.msg_iov = (struct iovec *)iovec;
+	hdr.msg_iovlen = iovec_cnt;
+
+	return sendmsg(GNUTLS_POINTER_TO_INT(ptr), &hdr, 0);
 }
 #endif
 
@@ -159,18 +165,38 @@ system_read(gnutls_transport_ptr_t ptr, void *data, size_t data_size)
  **/
 int gnutls_system_recv_timeout(gnutls_transport_ptr_t ptr, unsigned int ms)
 {
-	fd_set rfds;
-	struct timeval tv;
 	int ret;
 	int fd = GNUTLS_POINTER_TO_INT(ptr);
+#ifndef _WIN32
+	int timeo;
+	struct pollfd pfd;
+
+	pfd.fd = fd;
+	pfd.events = POLLIN;
+	pfd.revents = 0;
+
+	if (ms == GNUTLS_INDEFINITE_TIMEOUT)
+		timeo = -1;
+	else
+		timeo = ms;
+	do {
+		ret = poll(&pfd, 1, timeo);
+	} while(ret == -1 && errno == EINTR);
+#else
+	fd_set rfds;
+	struct timeval _tv, *tv = NULL;
 
 	FD_ZERO(&rfds);
 	FD_SET(fd, &rfds);
 
-	tv.tv_sec = ms/1000;
-	tv.tv_usec = (ms % 1000) * 1000;
+	if (ms != GNUTLS_INDEFINITE_TIMEOUT) {
+		_tv.tv_sec = ms/1000;
+		_tv.tv_usec = (ms % 1000) * 1000;
+		tv = &_tv;
+	}
 
-	ret = select(fd + 1, &rfds, NULL, NULL, &tv);
+	ret = select(fd + 1, &rfds, NULL, NULL, tv);
+#endif
 	if (ret <= 0)
 		return ret;
 
@@ -291,7 +317,7 @@ static int gnutls_system_mutex_unlock(void **priv)
 
 #endif				/* NO_LOCKS */
 
-gnutls_time_func gnutls_time;
+gnutls_time_func gnutls_time = time;
 mutex_init_func gnutls_mutex_init = gnutls_system_mutex_init;
 mutex_deinit_func gnutls_mutex_deinit = gnutls_system_mutex_deinit;
 mutex_lock_func gnutls_mutex_lock = gnutls_system_mutex_lock;
@@ -318,7 +344,6 @@ int gnutls_system_global_init(void)
 	Crypt32_dll = crypto;
 #endif
 #endif
-	gnutls_time = time;
 	return 0;
 }
 
@@ -329,7 +354,6 @@ void gnutls_system_global_deinit(void)
 	FreeLibrary(Crypt32_dll);
 #endif
 #endif
-	gnutls_time = time;
 }
 
 
@@ -340,7 +364,7 @@ void gnutls_system_global_deinit(void)
  */
 int _gnutls_find_config_path(char *path, size_t max_size)
 {
-	const char *home_dir = secure_getenv("HOME");
+	const char *home_dir = getenv("HOME");
 
 	if (home_dir != NULL && home_dir[0] != 0) {
 		snprintf(path, max_size, "%s/" CONFIG_PATH, home_dir);
@@ -431,7 +455,6 @@ static
 int add_system_trust(gnutls_x509_trust_list_t list, unsigned int tl_flags,
 		     unsigned int tl_vflags)
 {
-	char path[GNUTLS_PATH_MAX];
 	unsigned int i;
 	int r = 0;
 
@@ -615,6 +638,8 @@ int _gnutls_ucs2_to_utf8(const void *data, size_t size,
 		size -= 2;
 	}
 
+	src_len = wcslen(data);
+
 	src = gnutls_malloc(size+2);
 	if (src == NULL)
 		return gnutls_assert_val(GNUTLS_E_MEMORY_ERROR);
@@ -631,8 +656,6 @@ int _gnutls_ucs2_to_utf8(const void *data, size_t size,
 	src[size] = 0;
 	src[size+1] = 0;
 
-	src_len = wcslen(src);
-
 	ret =
 	    WideCharToMultiByte(CP_UTF8, flags,
 				(void *) src, src_len, NULL, 0,
