@@ -24,13 +24,13 @@
 /* Functions that relate on PKCS12 packet parsing.
  */
 
-#include <gnutls_int.h>
+#include "gnutls_int.h"
 #include <libtasn1.h>
 
-#include <gnutls_datum.h>
-#include <gnutls_global.h>
-#include <gnutls_errors.h>
-#include <gnutls_num.h>
+#include <datum.h>
+#include <global.h>
+#include "errors.h"
+#include <num.h>
 #include <common.h>
 #include <x509_b64.h>
 #include "x509_int.h"
@@ -1042,7 +1042,7 @@ int gnutls_pkcs12_verify_mac(gnutls_pkcs12_t pkcs12, const char *pass)
 		return _gnutls_asn2err(result);
 	}
 
-	algo = gnutls_oid_to_digest(oid);
+	algo = _gnutls_x509_oid_to_mac(oid);
 	if (algo == GNUTLS_MAC_UNKNOWN) {
  unknown_mac:
 		gnutls_assert();
@@ -1682,22 +1682,27 @@ gnutls_pkcs12_simple_parse(gnutls_pkcs12_t p12,
 				}
 
 				if (memcmp(cert_id, key_id, cert_id_size) != 0) {	/* they don't match - skip the certificate */
-					_extra_certs =
-						gnutls_realloc_fast
-						(_extra_certs,
-						 sizeof(_extra_certs
-							[0]) *
-						 ++_extra_certs_len);
-					if (!_extra_certs) {
-						gnutls_assert();
-						ret =
-							GNUTLS_E_MEMORY_ERROR;
-						goto done;
+					if (extra_certs) {
+						_extra_certs =
+						    gnutls_realloc_fast
+						    (_extra_certs,
+						     sizeof(_extra_certs
+							    [0]) *
+						     ++_extra_certs_len);
+						if (!_extra_certs) {
+							gnutls_assert();
+							ret =
+							    GNUTLS_E_MEMORY_ERROR;
+							goto done;
+						}
+						_extra_certs
+						    [_extra_certs_len -
+						     1] = this_cert;
+						this_cert = NULL;
+					} else {
+						gnutls_x509_crt_deinit
+						    (this_cert);
 					}
-					_extra_certs
-						[_extra_certs_len -
-						 1] = this_cert;
-					this_cert = NULL;
 				} else {
 					if (chain && _chain_len == 0) {
 						_chain =
@@ -1778,10 +1783,8 @@ gnutls_pkcs12_simple_parse(gnutls_pkcs12_t p12,
 		gnutls_pkcs12_bag_deinit(bag);
 
 	if (ret < 0) {
-		if (*key) {
+		if (*key)
 			gnutls_x509_privkey_deinit(*key);
-			*key = NULL;
-		}
 		if (_extra_certs_len && _extra_certs != NULL) {
 			for (i = 0; i < _extra_certs_len; i++)
 				gnutls_x509_crt_deinit(_extra_certs[i]);
@@ -1867,7 +1870,7 @@ gnutls_pkcs12_mac_info(gnutls_pkcs12_t pkcs12, unsigned int *mac,
 		*oid = (char*)tmp.data;
 	}
 
-	algo = gnutls_oid_to_digest((char*)tmp.data);
+	algo = _gnutls_x509_oid_to_mac((char*)tmp.data);
 	if (algo == GNUTLS_MAC_UNKNOWN || mac_to_entry(algo) == NULL) {
 		gnutls_assert();
 		return GNUTLS_E_UNKNOWN_HASH_ALGORITHM;
