@@ -24,11 +24,11 @@
 /* Functions that relate on PKCS12 Bag packet parsing.
  */
 
-#include <gnutls_int.h>
+#include "gnutls_int.h"
 
-#include <gnutls_datum.h>
-#include <gnutls_global.h>
-#include <gnutls_errors.h>
+#include <datum.h>
+#include <global.h>
+#include "errors.h"
 #include <common.h>
 #include "x509_int.h"
 
@@ -91,8 +91,7 @@ void gnutls_pkcs12_bag_deinit(gnutls_pkcs12_bag_t bag)
  *
  * This function will return the bag's type.
  *
- * Returns: On error a negative error value (when casted as integer) or
- *   one of the #gnutls_pkcs12_bag_type_t enumerations.
+ * Returns: One of the #gnutls_pkcs12_bag_type_t enumerations.
  **/
 gnutls_pkcs12_bag_type_t
 gnutls_pkcs12_bag_get_type(gnutls_pkcs12_bag_t bag, int indx)
