@@ -24,12 +24,12 @@
 /* Functions that relate on PKCS7 certificate lists parsing.
  */
 
-#include <gnutls_int.h>
+#include "gnutls_int.h"
 #include <libtasn1.h>
 
-#include <gnutls_datum.h>
-#include <gnutls_global.h>
-#include <gnutls_errors.h>
+#include <datum.h>
+#include <global.h>
+#include "errors.h"
 #include <common.h>
 #include <x509_b64.h>
 #include <gnutls/abstract.h>
@@ -50,22 +50,23 @@ static const uint8_t one = 1;
  * data are copied (they are locally allocated) there.
  */
 static int
-_decode_pkcs7_signed_data(gnutls_pkcs7_t pkcs7)
+_decode_pkcs7_signed_data(ASN1_TYPE pkcs7, ASN1_TYPE * sdata)
 {
+	char oid[MAX_OID_SIZE];
 	ASN1_TYPE c2;
-	int len, result;
 	gnutls_datum_t tmp = {NULL, 0};
+	int len, result;
 
-	len = MAX_OID_SIZE - 1;
-	result = asn1_read_value(pkcs7->pkcs7, "contentType", pkcs7->encap_data_oid, &len);
+	len = sizeof(oid) - 1;
+	result = asn1_read_value(pkcs7, "contentType", oid, &len);
 	if (result != ASN1_SUCCESS) {
 		gnutls_assert();
 		return _gnutls_asn2err(result);
 	}
 
-	if (strcmp(pkcs7->encap_data_oid, SIGNED_DATA_OID) != 0) {
+	if (strcmp(oid, SIGNED_DATA_OID) != 0) {
 		gnutls_assert();
-		_gnutls_debug_log("Unknown PKCS7 Content OID '%s'\n", pkcs7->encap_data_oid);
+		_gnutls_debug_log("Unknown PKCS7 Content OID '%s'\n", oid);
 		return GNUTLS_E_UNKNOWN_PKCS_CONTENT_TYPE;
 	}
 
@@ -79,12 +80,16 @@ _decode_pkcs7_signed_data(gnutls_pkcs7_t pkcs7)
 	/* the Signed-data has been created, so
 	 * decode them.
 	 */
-	result = _gnutls_x509_read_value(pkcs7->pkcs7, "content", &tmp);
+	result = _gnutls_x509_read_value(pkcs7, "content", &tmp);
 	if (result < 0) {
 		gnutls_assert();
 		goto cleanup;
 	}
 
+	/* tmp, tmp_size hold the data and the size of the CertificateSet structure
+	 * actually the ANY stuff.
+	 */
+
 	/* Step 1. In case of a signed structure extract certificate set.
 	 */
 
@@ -96,65 +101,30 @@ _decode_pkcs7_signed_data(gnutls_pkcs7_t pkcs7)
 	}
 
 	/* read the encapsulated content */
-	len = MAX_OID_SIZE - 1;
-	result =
-	    asn1_read_value(c2, "encapContentInfo.eContentType", pkcs7->encap_data_oid, &len);
+	len = sizeof(oid) - 1;
+	result = asn1_read_value(c2, "encapContentInfo.eContentType", oid, &len);
 	if (result != ASN1_SUCCESS) {
 		gnutls_assert();
 		result = _gnutls_asn2err(result);
 		goto cleanup;
 	}
 
-	if (strcmp(pkcs7->encap_data_oid, PLAIN_DATA_OID) != 0
-	    && strcmp(pkcs7->encap_data_oid, DIGESTED_DATA_OID) != 0) {
-		_gnutls_debug_log
-		    ("Unknown PKCS#7 Encapsulated Content OID '%s'; treating as raw data\n",
-		     pkcs7->encap_data_oid);
+	if (strcmp(oid, PLAIN_DATA_OID) != 0 && strcmp(oid, DIGESTED_DATA_OID) != 0) {
+		gnutls_assert();
+		_gnutls_debug_log("Unknown or unexpected PKCS7 Encapsulated Content OID '%s'\n", oid);
+		result = GNUTLS_E_UNKNOWN_PKCS_CONTENT_TYPE;
+		goto cleanup;
 	}
 
-	/* Try reading as octet string according to rfc5652. If that fails, attempt
-	 * a raw read according to rfc2315 */
-	result = _gnutls_x509_read_string(c2, "encapContentInfo.eContent", &pkcs7->der_signed_data, ASN1_ETYPE_OCTET_STRING, 0);
-	if (result < 0) {
-		result = _gnutls_x509_read_value(c2, "encapContentInfo.eContent", &pkcs7->der_signed_data);
-		if (result < 0) {
-			pkcs7->der_signed_data.data = NULL;
-			pkcs7->der_signed_data.size = 0;
-		} else {
-			int tag_len, len_len;
-			unsigned char cls;
-			unsigned long tag;
+	*sdata = c2;
 
-			/* we skip the embedded element's tag and length - uncharted territorry - used by MICROSOFT_CERT_TRUST_LIST */
-			result = asn1_get_tag_der(pkcs7->der_signed_data.data, pkcs7->der_signed_data.size, &cls, &tag_len, &tag);
-			if (result != ASN1_SUCCESS) {
-				gnutls_assert();
-				result = _gnutls_asn2err(result);
-				goto cleanup;
-			}
-
-			result = asn1_get_length_der(pkcs7->der_signed_data.data+tag_len, pkcs7->der_signed_data.size-tag_len, &len_len);
-			if (result < 0) {
-				gnutls_assert();
-				result = GNUTLS_E_ASN1_DER_ERROR;
-				goto cleanup;
-			}
-
-			tag_len += len_len;
-			memmove(pkcs7->der_signed_data.data, &pkcs7->der_signed_data.data[tag_len], pkcs7->der_signed_data.size-tag_len);
-			pkcs7->der_signed_data.size-=tag_len;
-		}
-	}
-
-	pkcs7->signed_data = c2;
 	gnutls_free(tmp.data);
-
 	return 0;
 
- cleanup:
-	gnutls_free(tmp.data);
+      cleanup:
 	if (c2)
 		asn1_delete_structure(&c2);
+	gnutls_free(tmp.data);
 	return result;
 }
 
@@ -220,8 +190,6 @@ void gnutls_pkcs7_deinit(gnutls_pkcs7_t pkcs7)
 	if (pkcs7->signed_data)
 		asn1_delete_structure(&pkcs7->signed_data);
 
-	_gnutls_free_datum(&pkcs7->der_signed_data);
-
 	gnutls_free(pkcs7);
 }
 
@@ -287,7 +255,7 @@ gnutls_pkcs7_import(gnutls_pkcs7_t pkcs7, const gnutls_datum_t * data,
 
 	/* Decode the signed data.
 	 */
-	result = _decode_pkcs7_signed_data(pkcs7);
+	result = _decode_pkcs7_signed_data(pkcs7->pkcs7, &pkcs7->signed_data);
 	if (result < 0) {
 		gnutls_assert();
 		goto cleanup;
@@ -752,10 +720,21 @@ static int verify_hash_attr(gnutls_pkcs7_t pkcs7, const char *root,
 	hash_size = gnutls_hash_get_len(hash);
 
 	if (data == NULL || data->data == NULL) {
-		data = &pkcs7->der_signed_data;
+		ret = _gnutls_x509_read_value(pkcs7->signed_data, "encapContentInfo.eContent", &tmp);
+		if (ret < 0) {
+			if (ret == GNUTLS_E_ASN1_ELEMENT_NOT_FOUND)
+				ret = GNUTLS_E_REQUESTED_DATA_NOT_AVAILABLE;
+			gnutls_assert();
+			return ret;
+		}
+		data = &tmp;
 	}
 
 	ret = gnutls_hash_fast(hash, data->data, data->size, hash_output);
+
+	gnutls_free(tmp.data);
+	tmp.data = NULL;
+
 	if (ret < 0)
 		return gnutls_assert_val(ret);
 
@@ -781,8 +760,6 @@ static int verify_hash_attr(gnutls_pkcs7_t pkcs7, const char *root,
 
 			if (tmp2.size == hash_size && memcmp(hash_output, tmp2.data, tmp2.size) == 0) {
 				msg_digest_ok = 1;
-			} else {
-				gnutls_assert();
 			}
 		} else if (strcmp(oid, ATTR_CONTENT_TYPE) == 0) {
 			if (num_cont_types > 0) {
@@ -806,11 +783,10 @@ static int verify_hash_attr(gnutls_pkcs7_t pkcs7, const char *root,
 				goto cleanup;
 			}
 		}
-
-		gnutls_free(tmp.data);
-		tmp.data = NULL;
-		gnutls_free(tmp2.data);
-		tmp2.data = NULL;
+	 	gnutls_free(tmp.data);
+	 	tmp.data = NULL;
+	 	gnutls_free(tmp2.data);
+	 	tmp2.data = NULL;
 	}
 
 	if (msg_digest_ok)
@@ -853,7 +829,12 @@ static int figure_pkcs7_sigdata(gnutls_pkcs7_t pkcs7, const char *root,
 
 	/* We have no signedAttrs. Use the provided data, or the encapsulated */
 	if (data == NULL || data->data == NULL) {
-		return _gnutls_set_datum(sigdata, pkcs7->der_signed_data.data, pkcs7->der_signed_data.size);
+		ret = _gnutls_x509_read_value(pkcs7->signed_data, "encapContentInfo.eContent", sigdata);
+		if (ret < 0) {
+			gnutls_assert();
+			return gnutls_assert_val(ret);
+		}
+		return 0;
 	}
 
 	return _gnutls_set_datum(sigdata, data->data, data->size);
@@ -862,19 +843,16 @@ static int figure_pkcs7_sigdata(gnutls_pkcs7_t pkcs7, const char *root,
 /**
  * gnutls_pkcs7_get_embedded_data:
  * @pkcs7: should contain a gnutls_pkcs7_t type
- * @flags: must be zero or %GNUTLS_PKCS7_EDATA_GET_RAW
+ * @idx: the index of the signature info to get the data from
  * @data: will hold the embedded data in the provided structure
  *
  * This function will return the data embedded in the signature of
  * the PKCS7 structure. If no data are available then
  * %GNUTLS_E_REQUESTED_DATA_NOT_AVAILABLE will be returned.
  *
- * The returned data must be de-allocated using gnutls_free().
- *
- * Note, that this function returns the exact same data that are
- * authenticated. If the %GNUTLS_PKCS7_EDATA_GET_RAW flag is provided,
- * the returned data will be including the wrapping tag/value as
- * they are encoded in the structure.
+ * Note, that since a PKCS#7 structure may contain embedded data
+ * for each attached signature, this function accepts and index which
+ * corresponds to the signature index to get the data from.
  *
  * Returns: On success, %GNUTLS_E_SUCCESS (0) is returned, otherwise a
  *   negative error value. 
@@ -882,45 +860,44 @@ static int figure_pkcs7_sigdata(gnutls_pkcs7_t pkcs7, const char *root,
  * Since: 3.4.8
  **/
 int
-gnutls_pkcs7_get_embedded_data(gnutls_pkcs7_t pkcs7, unsigned flags,
-			       gnutls_datum_t *data)
+gnutls_pkcs7_get_embedded_data(gnutls_pkcs7_t pkcs7, unsigned idx, gnutls_datum_t *data)
 {
+	int count, ret;
+	gnutls_datum_t tmpdata = {NULL, 0};
+	gnutls_pkcs7_signature_info_st info;
+	char root[128];
+
+	memset(&info, 0, sizeof(info));
+
 	if (pkcs7 == NULL)
 		return GNUTLS_E_INVALID_REQUEST;
 
-	if (pkcs7->der_signed_data.size == 0)
-		return gnutls_assert_val(GNUTLS_E_REQUESTED_DATA_NOT_AVAILABLE);
-
-	if (flags & GNUTLS_PKCS7_EDATA_GET_RAW) {
-		if (pkcs7->signed_data == NULL)
-			return gnutls_assert_val(GNUTLS_E_REQUESTED_DATA_NOT_AVAILABLE);
-
-		return _gnutls_x509_read_value(pkcs7->signed_data, "encapContentInfo.eContent", data);
-	} else {
-		return _gnutls_set_datum(data, pkcs7->der_signed_data.data, pkcs7->der_signed_data.size);
+	ret = asn1_number_of_elements(pkcs7->signed_data, "signerInfos", &count);
+	if (ret != ASN1_SUCCESS || idx+1 > (unsigned)count) {
+		gnutls_assert();
+		return GNUTLS_E_REQUESTED_DATA_NOT_AVAILABLE;
 	}
-}
 
-/**
- * gnutls_pkcs7_get_embedded_data_oid:
- * @pkcs7: should contain a gnutls_pkcs7_t type
- *
- * This function will return the OID of the data embedded in the signature of
- * the PKCS7 structure. If no data are available then %NULL will be
- * returned. The returned value will be valid during the lifetime
- * of the @pkcs7 structure.
- *
- * Returns: On success, a pointer to an OID string, %NULL on error.
- *
- * Since: 3.5.5
- **/
-const char *
-gnutls_pkcs7_get_embedded_data_oid(gnutls_pkcs7_t pkcs7)
-{
-	if (pkcs7 == NULL || pkcs7->encap_data_oid[0] == 0)
-		return NULL;
+	ret = gnutls_pkcs7_get_signature_info(pkcs7, idx, &info);
+	if (ret < 0) {
+		gnutls_assert();
+		goto cleanup;
+	}
 
-	return pkcs7->encap_data_oid;
+	snprintf(root, sizeof(root), "signerInfos.?%u", idx + 1);
+	ret = figure_pkcs7_sigdata(pkcs7, root, NULL, info.algo, data);
+	if (ret < 0) {
+		gnutls_assert();
+		goto cleanup;
+	}
+
+	ret = 0;
+
+ cleanup:
+	gnutls_free(tmpdata.data);
+	gnutls_pkcs7_signature_info_deinit(&info);
+
+	return ret;
 }
 
 /**
@@ -936,13 +913,6 @@ gnutls_pkcs7_get_embedded_data_oid(gnutls_pkcs7_t pkcs7)
  * provided are NULL then the data in the encapsulatedContent field
  * will be used instead.
  *
- * Note that, unlike gnutls_pkcs7_verify() this function does not
- * verify the key purpose of the signer. It is expected for the caller
- * to verify the intended purpose of the %signer -e.g., via gnutls_x509_crt_get_key_purpose_oid().
- *
- * Note also, that since GnuTLS 3.4.17 this function introduces checks in the
- * end certificate (@signer), including time checks and key usage checks.
- *
  * Returns: On success, %GNUTLS_E_SUCCESS (0) is returned, otherwise a
  *   negative error value. A verification error results to a
  *   %GNUTLS_E_PK_SIG_VERIFY_FAILED and the lack of encapsulated data
@@ -951,9 +921,10 @@ gnutls_pkcs7_get_embedded_data_oid(gnutls_pkcs7_t pkcs7)
  * Since: 3.4.2
  **/
 int gnutls_pkcs7_verify_direct(gnutls_pkcs7_t pkcs7,
-			       gnutls_x509_crt_t signer,
-			       unsigned idx,
-			       const gnutls_datum_t *data, unsigned flags)
+			gnutls_x509_crt_t signer,
+			unsigned idx,
+			const gnutls_datum_t *data,
+			unsigned flags)
 {
 	int count, ret;
 	gnutls_datum_t tmpdata = {NULL, 0};
@@ -1001,7 +972,6 @@ int gnutls_pkcs7_verify_direct(gnutls_pkcs7_t pkcs7,
 static
 gnutls_x509_crt_t find_signer(gnutls_pkcs7_t pkcs7, gnutls_x509_trust_list_t tl,
 			      gnutls_typed_vdata_st *vdata, unsigned vdata_size,
-			      unsigned vflags,
 			      gnutls_pkcs7_signature_info_st *info)
 {
 	gnutls_x509_crt_t issuer = NULL, crt = NULL;
@@ -1069,7 +1039,7 @@ gnutls_x509_crt_t find_signer(gnutls_pkcs7_t pkcs7, gnutls_x509_trust_list_t tl,
 
 	for (i=0;i<(unsigned)count;i++) {
 		/* Try to find the signer in the appended list. */
-		ret = gnutls_pkcs7_get_crt_raw2(pkcs7, i, &tmp);
+		ret = gnutls_pkcs7_get_crt_raw2(pkcs7, 0, &tmp);
 		if (ret < 0) {
 			gnutls_assert();
 			goto fail;
@@ -1099,7 +1069,7 @@ gnutls_x509_crt_t find_signer(gnutls_pkcs7_t pkcs7, gnutls_x509_trust_list_t tl,
 			goto skip;
 		}
 
-		ret = gnutls_x509_trust_list_verify_crt2(tl, &crt, 1, vdata, vdata_size, vflags, &vtmp, NULL);
+		ret = gnutls_x509_trust_list_verify_crt2(tl, &crt, 1, vdata, vdata_size, 0, &vtmp, NULL);
 		if (ret < 0 || vtmp != 0) {
 			gnutls_assert(); /* maybe next one is trusted */
  skip:
@@ -1155,10 +1125,11 @@ gnutls_x509_crt_t find_signer(gnutls_pkcs7_t pkcs7, gnutls_x509_trust_list_t tl,
  **/
 int gnutls_pkcs7_verify(gnutls_pkcs7_t pkcs7,
 			gnutls_x509_trust_list_t tl,
-			gnutls_typed_vdata_st *vdata,
+			gnutls_typed_vdata_st * vdata,
 			unsigned int vdata_size,
 			unsigned idx,
-			const gnutls_datum_t *data, unsigned flags)
+			const gnutls_datum_t *data,
+			unsigned flags)
 {
 	int count, ret;
 	gnutls_datum_t tmpdata = {NULL, 0};
@@ -1192,11 +1163,9 @@ int gnutls_pkcs7_verify(gnutls_pkcs7_t pkcs7,
 		goto cleanup;
 	}
 
-	signer = find_signer(pkcs7, tl, vdata, vdata_size, flags, &info);
+	signer = find_signer(pkcs7, tl, vdata, vdata_size, &info);
 	if (signer) {
-		ret =
-		    gnutls_x509_crt_verify_data3(signer, info.algo, vdata, vdata_size,
-						 &sigdata, &info.sig, flags);
+		ret = gnutls_x509_crt_verify_data2(signer, info.algo, flags, &sigdata, &info.sig);
 		if (ret < 0) {
 			gnutls_assert();
 		}
@@ -2105,12 +2074,10 @@ int gnutls_pkcs7_sign(gnutls_pkcs7_t pkcs7,
 		goto cleanup;
 	}
 
-	if (flags & GNUTLS_PKCS7_EMBED_DATA && data->data) {	/* embed data */
-		ret =
-		    _gnutls_x509_write_string(pkcs7->signed_data,
-				     "encapContentInfo.eContent", data,
-				     ASN1_ETYPE_OCTET_STRING);
-		if (ret < 0) {
+	if (flags & GNUTLS_PKCS7_EMBED_DATA && data->data) { /* embed data */
+		result = asn1_write_value(pkcs7->signed_data, "encapContentInfo.eContent", data->data, data->size);
+		if (result != ASN1_SUCCESS) {
+			ret = _gnutls_asn2err(result);
 			goto cleanup;
 		}
 	}
