@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2003-2012 Free Software Foundation, Inc.
+ * Copyright (C) 2012-2015 Nikos Mavrogiannopoulos
  *
  * Author: Nikos Mavrogiannopoulos
  *
@@ -20,18 +21,18 @@
  *
  */
 
-#include <gnutls_int.h>
-#include <gnutls_datum.h>
-#include <gnutls_global.h>
-#include <gnutls_errors.h>
-#include <gnutls_sig.h>
+#include "gnutls_int.h"
+#include <datum.h>
+#include <global.h>
+#include "errors.h"
+#include <tls-sig.h>
 #include <common.h>
-#include <gnutls_x509.h>
+#include <x509.h>
 #include <x509_b64.h>
 #include <x509_int.h>
-#include <gnutls_pk.h>
-#include <gnutls_mpi.h>
-#include <gnutls_ecc.h>
+#include <pk.h>
+#include <mpi.h>
+#include <ecc.h>
 #include <pin.h>
 
 /**
@@ -116,7 +117,7 @@ gnutls_x509_privkey_cpy(gnutls_x509_privkey_t dst,
 
 	ret =
 	    _gnutls_asn1_encode_privkey(dst->pk_algorithm, &dst->key,
-					&dst->params);
+					&dst->params, src->flags&GNUTLS_PRIVKEY_FLAG_EXPORT_COMPAT);
 	if (ret < 0) {
 		gnutls_assert();
 		return ret;
@@ -134,6 +135,8 @@ _gnutls_privkey_decode_pkcs1_rsa_key(const gnutls_datum_t * raw_key,
 {
 	int result;
 	ASN1_TYPE pkey_asn;
+	char tmp[64];
+	int tmp_size;
 
 	gnutls_pk_params_init(&pkey->params);
 
@@ -228,6 +231,29 @@ _gnutls_privkey_decode_pkcs1_rsa_key(const gnutls_datum_t * raw_key,
 
 	pkey->params.params_nr = RSA_PRIVATE_PARAMS;
 
+	tmp_size = sizeof(tmp);
+	result = asn1_read_value(pkey_asn, "otherInfo", tmp, &tmp_size);
+	if (result == ASN1_SUCCESS && strcmp(tmp, "seed") == 0) {
+		gnutls_datum_t v;
+		char oid[MAX_OID_SIZE];
+		int oid_size;
+
+		oid_size = sizeof(oid);
+		result = asn1_read_value(pkey_asn, "otherInfo.seed.algorithm", oid, &oid_size);
+		if (result == ASN1_SUCCESS) {
+			pkey->params.palgo = gnutls_oid_to_digest(oid);
+		}
+
+		result = _gnutls_x509_read_value(pkey_asn, "otherInfo.seed.seed", &v);
+		if (result >= 0) {
+			if (v.size <= sizeof(pkey->params.seed)) {
+				memcpy(pkey->params.seed, v.data, v.size);
+				pkey->params.seed_size = v.size;
+			}
+			gnutls_free(v.data);
+		}
+	}
+
 	return pkey_asn;
 
       error:
@@ -356,6 +382,9 @@ decode_dsa_key(const gnutls_datum_t * raw_key, gnutls_x509_privkey_t pkey)
 {
 	int result;
 	ASN1_TYPE dsa_asn;
+	gnutls_datum_t seed = {NULL,0};
+	char oid[MAX_OID_SIZE];
+	int oid_size;
 
 	if ((result =
 	     asn1_create_element(_gnutls_get_gnutls_asn(),
@@ -417,6 +446,21 @@ decode_dsa_key(const gnutls_datum_t * raw_key, gnutls_x509_privkey_t pkey)
 	}
 	pkey->params.params_nr++;
 
+	oid_size = sizeof(oid);
+	result = asn1_read_value(dsa_asn, "seed.algorithm", oid, &oid_size);
+	if (result == ASN1_SUCCESS) {
+		pkey->params.palgo = gnutls_oid_to_digest(oid);
+
+		result = _gnutls_x509_read_value(dsa_asn, "seed.seed", &seed);
+		if (result == ASN1_SUCCESS) {
+			if (seed.size <= sizeof(pkey->params.seed)) {
+				memcpy(pkey->params.seed, seed.data, seed.size);
+				pkey->params.seed_size = seed.size;
+			}
+			gnutls_free(seed.data);
+		}
+	}
+
 	return dsa_asn;
 
       error:
@@ -430,9 +474,20 @@ decode_dsa_key(const gnutls_datum_t * raw_key, gnutls_x509_privkey_t pkey)
 
 #define PEM_KEY_DSA "DSA PRIVATE KEY"
 #define PEM_KEY_RSA "RSA PRIVATE KEY"
+#define PEM_KEY_DSA_PROVABLE "FIPS186-4 DSA PRIVATE KEY"
+#define PEM_KEY_RSA_PROVABLE "FIPS186-4 RSA PRIVATE KEY"
 #define PEM_KEY_ECC "EC PRIVATE KEY"
 #define PEM_KEY_PKCS8 "PRIVATE KEY"
 
+#define MAX_PEM_HEADER_SIZE 25
+
+#define IF_CHECK_FOR(pemstr, algo, cptr, bptr, size, key) \
+		if (left > sizeof(pemstr) && memcmp(cptr, pemstr, sizeof(pemstr)-1) == 0) { \
+			result = _gnutls_fbase64_decode(pemstr, bptr, size, &_data); \
+			if (result >= 0) \
+				key->pk_algorithm = algo; \
+		}
+
 /**
  * gnutls_x509_privkey_import:
  * @key: The data to store the parsed key
@@ -482,9 +537,9 @@ gnutls_x509_privkey_import(gnutls_x509_privkey_t key,
 		if (ptr != NULL) {
 			left = data->size - ((ptrdiff_t)ptr - (ptrdiff_t)data->data);
 
-			if (data->size - left > 15) {
-				ptr -= 15;
-				left += 15;
+			if (data->size - left > MAX_PEM_HEADER_SIZE) {
+				ptr -= MAX_PEM_HEADER_SIZE;
+				left += MAX_PEM_HEADER_SIZE;
 			} else {
 				ptr = (char*)data->data;
 				left = data->size;
@@ -496,40 +551,22 @@ gnutls_x509_privkey_import(gnutls_x509_privkey_t key,
 				left = data->size - ((ptrdiff_t)begin_ptr - (ptrdiff_t)data->data);
 
 				ptr += sizeof("-----BEGIN ")-1;
-			}
 
-			if (ptr != NULL && left > sizeof(PEM_KEY_RSA)) {
-				if (memcmp(ptr, PEM_KEY_RSA, sizeof(PEM_KEY_RSA)-1) == 0) {
-					result =
-					    _gnutls_fbase64_decode(PEM_KEY_RSA, begin_ptr,
-							   	   left, &_data);
-					if (result >= 0)
-						key->pk_algorithm = GNUTLS_PK_RSA;
-				} else if (memcmp(ptr, PEM_KEY_ECC, sizeof(PEM_KEY_ECC)-1) == 0) {
-					result =
-					    _gnutls_fbase64_decode(PEM_KEY_ECC,
-								   begin_ptr,
-								   left,
-								   &_data);
-					if (result >= 0)
-						key->pk_algorithm = GNUTLS_PK_EC;
-				} else if (memcmp(ptr, PEM_KEY_DSA, sizeof(PEM_KEY_DSA)-1) == 0) {
-					result =
-					    _gnutls_fbase64_decode(PEM_KEY_DSA, begin_ptr,
-								   left, &_data);
-					if (result >= 0)
-						key->pk_algorithm = GNUTLS_PK_DSA;
-				}
-			}
+				IF_CHECK_FOR(PEM_KEY_RSA, GNUTLS_PK_RSA, ptr, begin_ptr, left, key)
+				else IF_CHECK_FOR(PEM_KEY_ECC, GNUTLS_PK_EC, ptr, begin_ptr, left, key)
+				else IF_CHECK_FOR(PEM_KEY_DSA, GNUTLS_PK_DSA, ptr, begin_ptr, left, key)
+				else IF_CHECK_FOR(PEM_KEY_RSA_PROVABLE, GNUTLS_PK_RSA, ptr, begin_ptr, left, key)
+				else IF_CHECK_FOR(PEM_KEY_DSA_PROVABLE, GNUTLS_PK_DSA, ptr, begin_ptr, left, key)
 
-			if (key->pk_algorithm == GNUTLS_PK_UNKNOWN && ptr != NULL && left >= sizeof(PEM_KEY_PKCS8)) {
-				if (memcmp(ptr, PEM_KEY_PKCS8, sizeof(PEM_KEY_PKCS8)-1) == 0) {
-					result =
-					    _gnutls_fbase64_decode(PEM_KEY_PKCS8, begin_ptr,
+				if (key->pk_algorithm == GNUTLS_PK_UNKNOWN && left >= sizeof(PEM_KEY_PKCS8)) {
+					if (memcmp(ptr, PEM_KEY_PKCS8, sizeof(PEM_KEY_PKCS8)-1) == 0) {
+						result =
+						    _gnutls_fbase64_decode(PEM_KEY_PKCS8, begin_ptr,
 							   	   left, &_data);
-					if (result >= 0) {
-						/* signal for PKCS #8 keys */
-						key->pk_algorithm = -1;
+						if (result >= 0) {
+							/* signal for PKCS #8 keys */
+							key->pk_algorithm = -1;
+						}
 					}
 				}
 			}
@@ -733,6 +770,7 @@ gnutls_x509_privkey_import2(gnutls_x509_privkey_t key,
 
 		if (ret < 0) {
 			gnutls_assert();
+			/* fall through to PKCS #8 decoding */
 		}
 	}
 
@@ -936,7 +974,7 @@ gnutls_x509_privkey_import_rsa_raw2(gnutls_x509_privkey_t key,
 
 	ret =
 	    _gnutls_asn1_encode_privkey(GNUTLS_PK_RSA, &key->key,
-					&key->params);
+					&key->params, key->flags&GNUTLS_PRIVKEY_FLAG_EXPORT_COMPAT);
 	if (ret < 0) {
 		gnutls_assert();
 		goto cleanup;
@@ -1024,7 +1062,7 @@ gnutls_x509_privkey_import_dsa_raw(gnutls_x509_privkey_t key,
 
 	ret =
 	    _gnutls_asn1_encode_privkey(GNUTLS_PK_DSA, &key->key,
-					&key->params);
+					&key->params, key->flags&GNUTLS_PRIVKEY_FLAG_EXPORT_COMPAT);
 	if (ret < 0) {
 		gnutls_assert();
 		goto cleanup;
@@ -1167,11 +1205,17 @@ gnutls_x509_privkey_get_pk_algorithm2(gnutls_x509_privkey_t key,
 
 static const char *set_msg(gnutls_x509_privkey_t key)
 {
-	if (key->pk_algorithm == GNUTLS_PK_RSA)
-		return PEM_KEY_RSA;
-	else if (key->pk_algorithm == GNUTLS_PK_DSA)
-		return PEM_KEY_DSA;
-	else if (key->pk_algorithm == GNUTLS_PK_EC)
+	if (key->pk_algorithm == GNUTLS_PK_RSA) {
+		if (key->params.seed_size > 0 && !(key->flags&GNUTLS_PRIVKEY_FLAG_EXPORT_COMPAT))
+			return PEM_KEY_RSA_PROVABLE;
+		else
+			return PEM_KEY_RSA;
+	} else if (key->pk_algorithm == GNUTLS_PK_DSA) {
+		if (key->params.seed_size > 0 && !(key->flags&GNUTLS_PRIVKEY_FLAG_EXPORT_COMPAT))
+			return PEM_KEY_DSA_PROVABLE;
+		else
+			return PEM_KEY_DSA;
+	} else if (key->pk_algorithm == GNUTLS_PK_EC)
 		return PEM_KEY_ECC;
 	else
 		return "UNKNOWN";
@@ -1205,6 +1249,7 @@ gnutls_x509_privkey_export(gnutls_x509_privkey_t key,
 			   size_t * output_data_size)
 {
 	const char *msg;
+	int ret;
 
 	if (key == NULL) {
 		gnutls_assert();
@@ -1213,6 +1258,12 @@ gnutls_x509_privkey_export(gnutls_x509_privkey_t key,
 
 	msg = set_msg(key);
 
+	if (key->flags & GNUTLS_PRIVKEY_FLAG_EXPORT_COMPAT) {
+		ret = gnutls_x509_privkey_fix(key);
+		if (ret < 0)
+			return gnutls_assert_val(ret);
+	}
+
 	return _gnutls_x509_export_int(key->key, format, msg,
 				       output_data, output_data_size);
 }
@@ -1243,6 +1294,7 @@ gnutls_x509_privkey_export2(gnutls_x509_privkey_t key,
 			    gnutls_datum_t * out)
 {
 	const char *msg;
+	int ret;
 
 	if (key == NULL) {
 		gnutls_assert();
@@ -1251,6 +1303,12 @@ gnutls_x509_privkey_export2(gnutls_x509_privkey_t key,
 
 	msg = set_msg(key);
 
+	if (key->flags & GNUTLS_PRIVKEY_FLAG_EXPORT_COMPAT) {
+		ret = gnutls_x509_privkey_fix(key);
+		if (ret < 0)
+			return gnutls_assert_val(ret);
+	}
+
 	return _gnutls_x509_export_int2(key->key, format, msg, out);
 }
 
@@ -1482,13 +1540,50 @@ cleanup:
 
 /**
  * gnutls_x509_privkey_generate:
+ * @key: an initialized key
+ * @algo: is one of the algorithms in #gnutls_pk_algorithm_t.
+ * @bits: the size of the parameters to generate
+ * @flags: Must be zero or flags from #gnutls_privkey_flags_t.
+ *
+ * This function will generate a random private key. Note that this
+ * function must be called on an empty private key. The flag %GNUTLS_PRIVKEY_FLAG_PROVABLE
+ * instructs the key generation process to use algorithms like Shawe-Taylor
+ * which generate provable parameters out of a seed.
+ *
+ * Note that when generating an elliptic curve key, the curve
+ * can be substituted in the place of the bits parameter using the
+ * GNUTLS_CURVE_TO_BITS() macro. The input to the macro is any curve from
+ * %gnutls_ecc_curve_t.
+ *
+ * For DSA keys, if the subgroup size needs to be specified check
+ * the GNUTLS_SUBGROUP_TO_BITS() macro.
+ *
+ * It is recommended to do not set the number of @bits directly, use gnutls_sec_param_to_pk_bits() instead .
+ *
+ * Returns: On success, %GNUTLS_E_SUCCESS (0) is returned, otherwise a
+ *   negative error value.
+ **/
+int
+gnutls_x509_privkey_generate(gnutls_x509_privkey_t key,
+			     gnutls_pk_algorithm_t algo, unsigned int bits,
+			     unsigned int flags)
+{
+	return gnutls_x509_privkey_generate2(key, algo, bits, flags, NULL, 0);   
+}
+
+/**
+ * gnutls_x509_privkey_generate2:
  * @key: a key
  * @algo: is one of the algorithms in #gnutls_pk_algorithm_t.
  * @bits: the size of the modulus
- * @flags: unused for now.  Must be 0.
+ * @flags: Must be zero or flags from #gnutls_privkey_flags_t.
+ * @data: Allow specifying %gnutls_keygen_data_st types such as the seed to be used.
+ * @data_size: The number of @data available.
  *
  * This function will generate a random private key. Note that this
- * function must be called on an empty private key.
+ * function must be called on an empty private key. The flag %GNUTLS_PRIVKEY_FLAG_PROVABLE
+ * instructs the key generation process to use algorithms which generate
+ * provable parameters out of a seed.
  *
  * Note that when generating an elliptic curve key, the curve
  * can be substituted in the place of the bits parameter using the
@@ -1503,11 +1598,12 @@ cleanup:
  *   negative error value.
  **/
 int
-gnutls_x509_privkey_generate(gnutls_x509_privkey_t key,
-			     gnutls_pk_algorithm_t algo, unsigned int bits,
-			     unsigned int flags)
+gnutls_x509_privkey_generate2(gnutls_x509_privkey_t key,
+			      gnutls_pk_algorithm_t algo, unsigned int bits,
+			      unsigned int flags, const gnutls_keygen_data_st *data, unsigned data_size)
 {
 	int ret;
+	unsigned i;
 
 	if (key == NULL) {
 		gnutls_assert();
@@ -1516,6 +1612,15 @@ gnutls_x509_privkey_generate(gnutls_x509_privkey_t key,
 
 	gnutls_pk_params_init(&key->params);
 
+	for (i=0;i<data_size;i++) {
+		if (data[i].type == GNUTLS_KEYGEN_SEED && data[i].size < sizeof(key->params.seed)) {
+			key->params.seed_size = data[i].size;
+			memcpy(key->params.seed, data[i].data, data[i].size);
+		} else if (data[i].type == GNUTLS_KEYGEN_DIGEST) {
+			key->params.palgo = data[i].size;
+		}
+	}
+
 	if (algo == GNUTLS_PK_EC) {
 		if (GNUTLS_BITS_ARE_CURVE(bits))
 			bits = GNUTLS_BITS_TO_CURVE(bits);
@@ -1523,6 +1628,10 @@ gnutls_x509_privkey_generate(gnutls_x509_privkey_t key,
 			bits = _gnutls_ecc_bits_to_curve(bits);
 	}
 
+	if (flags & GNUTLS_PRIVKEY_FLAG_PROVABLE) {
+		key->params.flags |= GNUTLS_PK_FLAG_PROVABLE;
+	}
+
 	ret = _gnutls_pk_generate_params(algo, bits, &key->params);
 	if (ret < 0) {
 		gnutls_assert();
@@ -1534,7 +1643,6 @@ gnutls_x509_privkey_generate(gnutls_x509_privkey_t key,
 		gnutls_assert();
 		goto cleanup;
 	}
-
 #ifndef ENABLE_FIPS140
 	ret = _gnutls_pk_verify_priv_params(algo, &key->params);
 #else
@@ -1545,7 +1653,7 @@ gnutls_x509_privkey_generate(gnutls_x509_privkey_t key,
 		goto cleanup;
 	}
 
-	ret = _gnutls_asn1_encode_privkey(algo, &key->key, &key->params);
+	ret = _gnutls_asn1_encode_privkey(algo, &key->key, &key->params, key->flags&GNUTLS_PRIVKEY_FLAG_EXPORT_COMPAT);
 	if (ret < 0) {
 		gnutls_assert();
 		goto cleanup;
@@ -1562,6 +1670,228 @@ gnutls_x509_privkey_generate(gnutls_x509_privkey_t key,
 	return ret;
 }
 
+/**
+ * gnutls_x509_privkey_get_seed:
+ * @key: should contain a #gnutls_x509_privkey_t type
+ * @digest: if non-NULL it will contain the digest algorithm used for key generation (if applicable)
+ * @seed: where seed will be copied to
+ * @seed_size: originally holds the size of @seed, will be updated with actual size
+ *
+ * This function will return the seed that was used to generate the
+ * given private key. That function will succeed only if the key was generated
+ * as a provable key.
+ *
+ * Returns: On success, %GNUTLS_E_SUCCESS (0) is returned, otherwise a
+ *   negative error value.
+ *
+ * Since: 3.5.0
+ **/
+int gnutls_x509_privkey_get_seed(gnutls_x509_privkey_t key, gnutls_digest_algorithm_t *digest, void *seed, size_t *seed_size)
+{
+	if (key->params.seed_size == 0)
+		return gnutls_assert_val(GNUTLS_E_INVALID_REQUEST);
+
+	if (seed_size == NULL || seed == NULL) {
+		if (key->params.seed_size)
+			return gnutls_assert_val(GNUTLS_E_SHORT_MEMORY_BUFFER);
+		else
+			return gnutls_assert_val(GNUTLS_E_INVALID_REQUEST);
+	}
+
+	if (*seed_size < key->params.seed_size) {
+		*seed_size = key->params.seed_size;
+		return gnutls_assert_val(GNUTLS_E_SHORT_MEMORY_BUFFER);
+	}
+
+	if (digest)
+		*digest = key->params.palgo;
+
+	memcpy(seed, key->params.seed, key->params.seed_size);
+	*seed_size = key->params.seed_size; 
+	return 0;
+}
+
+static
+int cmp_rsa_key(gnutls_x509_privkey_t key1, gnutls_x509_privkey_t key2)
+{
+	gnutls_datum_t m1 = {NULL, 0}, e1 = {NULL, 0}, d1 = {NULL, 0}, p1 = {NULL, 0}, q1 = {NULL, 0};
+	gnutls_datum_t m2 = {NULL, 0}, e2 = {NULL, 0}, d2 = {NULL, 0}, p2 = {NULL, 0}, q2 = {NULL, 0};
+	int ret;
+
+	ret = gnutls_x509_privkey_export_rsa_raw(key1, &m1, &e1, &d1, &p1, &q1, NULL);
+	if (ret < 0) {
+		gnutls_assert();
+		return ret;
+	}
+
+	ret = gnutls_x509_privkey_export_rsa_raw(key2, &m2, &e2, &d2, &p2, &q2, NULL);
+	if (ret < 0) {
+		gnutls_assert();
+		goto cleanup;
+	}
+
+	if (m1.size != m2.size || memcmp(m1.data, m2.data, m1.size) != 0) {
+		gnutls_assert();
+		ret = GNUTLS_E_PRIVKEY_VERIFICATION_ERROR;
+		goto cleanup;
+	}
+
+	if (d1.size != d2.size || memcmp(d1.data, d2.data, d1.size) != 0) {
+		gnutls_assert();
+		ret = GNUTLS_E_PRIVKEY_VERIFICATION_ERROR;
+		goto cleanup;
+	}
+
+	if (e1.size != e2.size || memcmp(e1.data, e2.data, e1.size) != 0) {
+		gnutls_assert();
+		ret = GNUTLS_E_PRIVKEY_VERIFICATION_ERROR;
+		goto cleanup;
+	}
+
+	if (p1.size != p2.size || memcmp(p1.data, p2.data, p1.size) != 0) {
+		gnutls_assert();
+		ret = GNUTLS_E_PRIVKEY_VERIFICATION_ERROR;
+		goto cleanup;
+	}
+
+	if (q1.size != q2.size || memcmp(q1.data, q2.data, q1.size) != 0) {
+		gnutls_assert();
+		ret = GNUTLS_E_PRIVKEY_VERIFICATION_ERROR;
+		goto cleanup;
+	}
+
+	ret = 0;
+ cleanup:
+      	gnutls_free(m1.data);
+      	gnutls_free(e1.data);
+      	gnutls_free(d1.data);
+      	gnutls_free(p1.data);
+      	gnutls_free(q1.data);
+      	gnutls_free(m2.data);
+      	gnutls_free(e2.data);
+      	gnutls_free(d2.data);
+      	gnutls_free(p2.data);
+      	gnutls_free(q2.data);
+ 	return ret;
+}
+
+static
+int cmp_dsa_key(gnutls_x509_privkey_t key1, gnutls_x509_privkey_t key2)
+{
+	gnutls_datum_t p1 = {NULL, 0}, q1 = {NULL, 0}, g1 = {NULL, 0};
+	gnutls_datum_t p2 = {NULL, 0}, q2 = {NULL, 0}, g2 = {NULL, 0};
+	int ret;
+
+	ret = gnutls_x509_privkey_export_dsa_raw(key1, &p1, &q1, &g1, NULL, NULL);
+	if (ret < 0) {
+		gnutls_assert();
+		return ret;
+	}
+
+	ret = gnutls_x509_privkey_export_dsa_raw(key2, &p2, &q2, &g2, NULL, NULL);
+	if (ret < 0) {
+		gnutls_assert();
+		goto cleanup;
+	}
+
+	if (g1.size != g2.size || memcmp(g1.data, g2.data, g1.size) != 0) {
+		gnutls_assert();
+		ret = GNUTLS_E_PRIVKEY_VERIFICATION_ERROR;
+		goto cleanup;
+	}
+
+	if (p1.size != p2.size || memcmp(p1.data, p2.data, p1.size) != 0) {
+		gnutls_assert();
+		ret = GNUTLS_E_PRIVKEY_VERIFICATION_ERROR;
+		goto cleanup;
+	}
+
+	if (q1.size != q2.size || memcmp(q1.data, q2.data, q1.size) != 0) {
+		gnutls_assert();
+		ret = GNUTLS_E_PRIVKEY_VERIFICATION_ERROR;
+		goto cleanup;
+	}
+
+	ret = 0;
+ cleanup:
+      	gnutls_free(g1.data);
+      	gnutls_free(p1.data);
+      	gnutls_free(q1.data);
+      	gnutls_free(g2.data);
+      	gnutls_free(p2.data);
+      	gnutls_free(q2.data);
+ 	return ret;
+}
+
+/**
+ * gnutls_x509_privkey_verify_seed:
+ * @key: should contain a #gnutls_x509_privkey_t type
+ * @digest: it contains the digest algorithm used for key generation (if applicable)
+ * @seed: the seed of the key to be checked with
+ * @seed_size: holds the size of @seed
+ *
+ * This function will verify that the given private key was generated from
+ * the provided seed. If @seed is %NULL then the seed stored in the @key's structure
+ * will be used for verification.
+ *
+ * Returns: In case of a verification failure %GNUTLS_E_PRIVKEY_VERIFICATION_ERROR
+ * is returned, and zero or positive code on success.
+ *
+ * Since: 3.5.0
+ **/
+int gnutls_x509_privkey_verify_seed(gnutls_x509_privkey_t key, gnutls_digest_algorithm_t digest, const void *seed, size_t seed_size)
+{
+	int ret;
+	gnutls_x509_privkey_t okey;
+	unsigned bits;
+	gnutls_keygen_data_st data;
+
+	if (key == NULL) {
+		gnutls_assert();
+		return GNUTLS_E_INVALID_REQUEST;
+	}
+
+	if (key->pk_algorithm != GNUTLS_PK_RSA && key->pk_algorithm != GNUTLS_PK_DSA)
+		return gnutls_assert_val(GNUTLS_E_INVALID_REQUEST);
+
+	ret = gnutls_x509_privkey_get_pk_algorithm2(key, &bits);
+	if (ret < 0)
+		return gnutls_assert_val(ret);
+
+	ret = gnutls_x509_privkey_init(&okey);
+	if (ret < 0)
+		return gnutls_assert_val(ret);
+
+	if (seed == NULL) {
+		seed = key->params.seed;
+		seed_size = key->params.seed_size;
+	}
+
+	if (seed == NULL || seed_size == 0)
+		return gnutls_assert_val(GNUTLS_E_INVALID_REQUEST);
+
+	data.type = GNUTLS_KEYGEN_SEED;
+	data.data = (void*)seed;
+	data.size = seed_size;
+
+	ret = gnutls_x509_privkey_generate2(okey, key->pk_algorithm, bits,
+					    GNUTLS_PRIVKEY_FLAG_PROVABLE, &data, 1);
+	if (ret < 0) {
+		gnutls_assert();
+		goto cleanup;
+	}
+
+	if (key->pk_algorithm == GNUTLS_PK_RSA)
+		ret = cmp_rsa_key(key, okey);
+	else
+		ret = cmp_dsa_key(key, okey);
+
+      cleanup:
+        gnutls_x509_privkey_deinit(okey);
+
+	return ret;
+}
+
 /**
  * gnutls_x509_privkey_verify_params:
  * @key: a key
@@ -1827,7 +2157,7 @@ int gnutls_x509_privkey_fix(gnutls_x509_privkey_t key)
 
 	ret =
 	    _gnutls_asn1_encode_privkey(key->pk_algorithm, &key->key,
-					&key->params);
+					&key->params, key->flags&GNUTLS_PRIVKEY_FLAG_EXPORT_COMPAT);
 	if (ret < 0) {
 		gnutls_assert();
 		return ret;
@@ -1858,3 +2188,21 @@ void gnutls_x509_privkey_set_pin_function(gnutls_x509_privkey_t privkey,
 	privkey->pin.cb = fn;
 	privkey->pin.data = userdata;
 }
+
+/**
+ * gnutls_x509_privkey_set_flags:
+ * @key: A key of type #gnutls_x509_privkey_t
+ * @flags: flags from the %gnutls_privkey_flags
+ *
+ * This function will set flags for the specified private key, after
+ * it is generated. Currently this is useful for the %GNUTLS_PRIVKEY_FLAG_EXPORT_COMPAT
+ * to allow exporting a "provable" private key in backwards compatible way.
+ *
+ * Since: 3.5.0
+ *
+ **/
+void gnutls_x509_privkey_set_flags(gnutls_x509_privkey_t key,
+ 			      	   unsigned int flags)
+{
+	key->flags |= flags;
+}
