@@ -24,15 +24,15 @@
  * included here
  */
 
-#include <gnutls_int.h>
+#include "gnutls_int.h"
 
-#include <gnutls_errors.h>
+#include "errors.h"
 #include <libtasn1.h>
-#include <gnutls_global.h>
-#include <gnutls_num.h>		/* MAX */
-#include <gnutls_sig.h>
-#include <gnutls_str.h>
-#include <gnutls_datum.h>
+#include <global.h>
+#include <num.h>		/* MAX */
+#include <tls-sig.h>
+#include <str.h>
+#include <datum.h>
 #include <x509_int.h>
 #include <common.h>
 #include <gnutls/abstract.h>
@@ -119,7 +119,7 @@ _gnutls_x509_pkix_sign(ASN1_TYPE src, const char *src_name,
 
 	result = _gnutls_x509_write_sig_params(src, name,
 					       gnutls_privkey_get_pk_algorithm
-					       (issuer_key, NULL), dig);
+					       (issuer_key, NULL), dig, 0);
 	if (result < 0) {
 		gnutls_assert();
 		return result;
@@ -162,7 +162,7 @@ _gnutls_x509_pkix_sign(ASN1_TYPE src, const char *src_name,
 
 	result = _gnutls_x509_write_sig_params(src, "signatureAlgorithm",
 					       gnutls_privkey_get_pk_algorithm
-					       (issuer_key, NULL), dig);
+					       (issuer_key, NULL), dig, 0);
 	if (result < 0) {
 		gnutls_assert();
 		return result;
