@@ -21,14 +21,14 @@
  *
  */
 
-#include <gnutls_int.h>
-#include <gnutls_errors.h>
+#include "gnutls_int.h"
+#include "errors.h"
 #include <libtasn1.h>
-#include <gnutls_global.h>
-#include <gnutls_num.h>		/* MAX */
-#include <gnutls_sig.h>
-#include <gnutls_str.h>
-#include <gnutls_datum.h>
+#include <global.h>
+#include <num.h>		/* MAX */
+#include <tls-sig.h>
+#include <str.h>
+#include <datum.h>
 #include <hash-pjw-bare.h>
 #include "x509_int.h"
 #include <common.h>
@@ -289,7 +289,7 @@ gnutls_x509_trust_list_add_cas(gnutls_x509_trust_list_t list,
 		if (flags & GNUTLS_TL_NO_DUPLICATES || flags & GNUTLS_TL_NO_DUPLICATE_KEY) {
 			for (j=0;j<list->node[hash].trusted_ca_size;j++) {
 				if (flags & GNUTLS_TL_NO_DUPLICATES)
-					ret = _gnutls_check_if_same_cert(list->node[hash].trusted_cas[j], clist[i]);
+					ret = gnutls_x509_crt_equals(list->node[hash].trusted_cas[j], clist[i]);
 				else
 					ret = _gnutls_check_if_same_key(list->node[hash].trusted_cas[j], clist[i], 1);
 				if (ret != 0) {
@@ -573,7 +573,7 @@ gnutls_x509_trust_list_remove_cas(gnutls_x509_trust_list_t list,
 		hash %= list->size;
 
 		for (j = 0; j < list->node[hash].trusted_ca_size; j++) {
-			if (_gnutls_check_if_same_cert
+			if (gnutls_x509_crt_equals
 			    (clist[i],
 			     list->node[hash].trusted_cas[j]) != 0) {
 
@@ -820,7 +820,7 @@ static int shorten_clist(gnutls_x509_trust_list_t list,
 		hash %= list->size;
 
 		for (j = 0; j < list->node[hash].trusted_ca_size; j++) {
-			if (_gnutls_check_if_same_cert
+			if (gnutls_x509_crt_equals
 			    (certificate_list[i],
 			     list->node[hash].trusted_cas[j]) != 0) {
 				/* cut the list at the point of first the trusted certificate */
@@ -1130,7 +1130,7 @@ unsigned i, j;
 
 	for (i=0;i<cert_list_size;i++) {
 		for (j=0;j<blacklist_size;j++) {
-			if (_gnutls_check_if_same_cert(cert_list[i], blacklist[j]) != 0) {
+			if (gnutls_x509_crt_equals(cert_list[i], blacklist[j]) != 0) {
 				return 1;
 			}
 		}
@@ -1453,7 +1453,7 @@ gnutls_x509_trust_list_verify_named_crt(gnutls_x509_trust_list_t list,
 	*voutput = GNUTLS_CERT_INVALID | GNUTLS_CERT_SIGNER_NOT_FOUND;
 
 	for (i = 0; i < list->node[hash].named_cert_size; i++) {
-		if (_gnutls_check_if_same_cert(cert, list->node[hash].named_certs[i].cert) != 0) {	/* check if name matches */
+		if (gnutls_x509_crt_equals(cert, list->node[hash].named_certs[i].cert) != 0) {	/* check if name matches */
 			if (list->node[hash].named_certs[i].name_size ==
 			    name_size
 			    && memcmp(list->node[hash].named_certs[i].name,
@@ -1497,7 +1497,7 @@ _gnutls_trustlist_inlist(gnutls_x509_trust_list_t list,
 
 	for (i = 0; i < list->node[hash].trusted_ca_size; i++) {
 		ret =
-		    _gnutls_check_if_same_cert(cert,
+		    gnutls_x509_crt_equals(cert,
 					       list->node[hash].
 					       trusted_cas[i]);
 		if (ret != 0)
