@@ -26,21 +26,21 @@
  * included here
  */
 
-#include <gnutls_int.h>
-#include <gnutls_errors.h>
+#include "gnutls_int.h"
+#include "errors.h"
 #include <libtasn1.h>
-#include <gnutls_global.h>
-#include <gnutls_num.h>		/* MAX */
-#include <gnutls_sig.h>
-#include <gnutls_str.h>
-#include <gnutls_datum.h>
+#include <global.h>
+#include <num.h>		/* MAX */
+#include <tls-sig.h>
+#include <str.h>
+#include <datum.h>
 #include <x509_int.h>
 #include <common.h>
-#include <gnutls_pk.h>
+#include <pk.h>
 #include <stdbool.h>
 
 /* Checks if two certs have the same name and the same key.  Return 1 on match. 
- * If @is_ca is zero then this function is identical to _gnutls_check_if_same_cert()
+ * If @is_ca is zero then this function is identical to gnutls_x509_crt_equals()
  */
 bool
 _gnutls_check_if_same_key(gnutls_x509_crt_t cert1,
@@ -51,7 +51,7 @@ _gnutls_check_if_same_key(gnutls_x509_crt_t cert1,
 	bool result;
 
 	if (is_ca == 0)
-		return _gnutls_check_if_same_cert(cert1, cert2);
+		return gnutls_x509_crt_equals(cert1, cert2);
 
 	ret = _gnutls_is_same_dn(cert1, cert2);
 	if (ret == 0)
@@ -89,40 +89,6 @@ _gnutls_check_if_same_key2(gnutls_x509_crt_t cert1,
 	return ret;
 }
 
-bool
-_gnutls_check_if_same_cert(gnutls_x509_crt_t cert1,
-			   gnutls_x509_crt_t cert2)
-{
-	int ret;
-	bool result;
-
-	ret = _gnutls_is_same_dn(cert1, cert2);
-	if (ret == 0)
-		return 0;
-
-	if ((cert1->der.size == cert2->der.size) &&
-	    (memcmp(cert1->der.data, cert2->der.data, cert1->der.size) == 0))
-		result = 1;
-	else
-		result = 0;
-
-	return result;
-}
-
-bool
-_gnutls_check_if_same_cert2(gnutls_x509_crt_t cert1,
-			    gnutls_datum_t * cert2bin)
-{
-	bool result;
-
-	if ((cert1->der.size == cert2bin->size) &&
-	    (memcmp(cert1->der.data, cert2bin->data, cert1->der.size) == 0))
-		result = 1;
-	else
-		result = 0;
-
-	return result;
-}
 
 /* Checks if the issuer of a certificate is a
  * Certificate Authority, or if the certificate is the same
@@ -423,7 +389,7 @@ int is_broken_allowed(gnutls_sign_algorithm_t sig, unsigned int flags)
  * according to verification profile specified.
  *
  * @crt: a certificate
- * @issuer: the certificates issuer
+ * @issuer: the certificates issuer (allowed to be NULL)
  * @sigalg: the signature algorithm used
  * @flags: the specified verification flags
  */
@@ -469,11 +435,6 @@ int hash;
 				return gnutls_assert_val(0);
 			}
 
-			if (gnutls_x509_crt_get_version(issuer) != 3) {
-				_gnutls_debug_log("SUITEB: certificate's issuer uses an unacceptable version number\n");
-				return gnutls_assert_val(0);
-			}
-
 			if (sigalg != GNUTLS_SIGN_ECDSA_SHA256 && sigalg != GNUTLS_SIGN_ECDSA_SHA384) {
 				_gnutls_debug_log("SUITEB: certificate is not signed using ECDSA-SHA256 or ECDSA-SHA384\n");
 				return gnutls_assert_val(0);
@@ -498,38 +459,12 @@ int hash;
 			curve = params.flags;
 			gnutls_pk_params_release(&params);
 
-			ret = _gnutls_x509_crt_get_mpis(issuer, &params);
-			if (ret < 0) {
-				_gnutls_debug_log("SUITEB: cannot read certificate params\n");
-				return gnutls_assert_val(0);
-			}
-
-			issuer_curve = params.flags;
-			gnutls_pk_params_release(&params);
-
-			if (issuer_curve != GNUTLS_ECC_CURVE_SECP256R1 &&
-				issuer_curve != GNUTLS_ECC_CURVE_SECP384R1) {
-				_gnutls_debug_log("SUITEB: certificate's issuer ECC params do not contain SECP256R1 or SECP384R1\n");
-				return gnutls_assert_val(0);
-			}
-
 			if (curve != GNUTLS_ECC_CURVE_SECP256R1 &&
 				curve != GNUTLS_ECC_CURVE_SECP384R1) {
 				_gnutls_debug_log("SUITEB: certificate's ECC params do not contain SECP256R1 or SECP384R1\n");
 				return gnutls_assert_val(0);
 			}
 
-			if (issuer_curve < curve) {
-				_gnutls_debug_log("SUITEB: certificate's issuer ECC params are weaker than the certificate's\n");
-				return gnutls_assert_val(0);
-			}
-
-			if (sigalg == GNUTLS_SIGN_ECDSA_SHA256 && 
-				issuer_curve == GNUTLS_ECC_CURVE_SECP384R1) {
-				_gnutls_debug_log("SUITEB: certificate is signed with ECDSA-SHA256 when using SECP384R1\n");
-				return gnutls_assert_val(0);
-			}
-
 			if (profile == GNUTLS_PROFILE_SUITEB192) {
 				if (curve != GNUTLS_ECC_CURVE_SECP384R1) {
 					_gnutls_debug_log("SUITEB192: certificate does not use SECP384R1\n");
@@ -537,6 +472,39 @@ int hash;
 				}
 			}
 
+			if (issuer != NULL) {
+				if (gnutls_x509_crt_get_version(issuer) != 3) {
+					_gnutls_debug_log("SUITEB: certificate's issuer uses an unacceptable version number\n");
+					return gnutls_assert_val(0);
+				}
+
+				ret = _gnutls_x509_crt_get_mpis(issuer, &params);
+				if (ret < 0) {
+					_gnutls_debug_log("SUITEB: cannot read certificate params\n");
+					return gnutls_assert_val(0);
+				}
+
+				issuer_curve = params.flags;
+				gnutls_pk_params_release(&params);
+
+				if (issuer_curve != GNUTLS_ECC_CURVE_SECP256R1 &&
+					issuer_curve != GNUTLS_ECC_CURVE_SECP384R1) {
+					_gnutls_debug_log("SUITEB: certificate's issuer ECC params do not contain SECP256R1 or SECP384R1\n");
+					return gnutls_assert_val(0);
+				}
+
+				if (issuer_curve < curve) {
+					_gnutls_debug_log("SUITEB: certificate's issuer ECC params are weaker than the certificate's\n");
+					return gnutls_assert_val(0);
+				}
+
+				if (sigalg == GNUTLS_SIGN_ECDSA_SHA256 && 
+					issuer_curve == GNUTLS_ECC_CURVE_SECP384R1) {
+					_gnutls_debug_log("SUITEB: certificate is signed with ECDSA-SHA256 when using SECP384R1\n");
+					return gnutls_assert_val(0);
+				}
+			}
+
 			break;
 		}
 	}
@@ -572,7 +540,7 @@ verify_crt(gnutls_x509_crt_t cert,
 	gnutls_datum_t cert_signature = { NULL, 0 };
 	gnutls_x509_crt_t issuer = NULL;
 	int issuer_version, hash_algo;
-	bool result = 0;
+	bool result = 1;
 	const mac_entry_st * me;
 	unsigned int out = 0, usage;
 	int sigalg, ret;
@@ -581,13 +549,12 @@ verify_crt(gnutls_x509_crt_t cert,
 		*output = 0;
 
 	if (*max_path == 0) {
-		out =
+		out |=
 		    GNUTLS_CERT_SIGNER_CONSTRAINTS_FAILURE |
 		    GNUTLS_CERT_INVALID;
-		if (output)
-			*output |= out;
 		gnutls_assert();
 		result = 0;
+		/* bail immediately, to avoid inconistency  */
 		goto cleanup;
 	}
 	(*max_path)--;
@@ -595,115 +562,16 @@ verify_crt(gnutls_x509_crt_t cert,
 	if (tcas_size >= 1)
 		issuer = find_issuer(cert, trusted_cas, tcas_size);
 
-	/* issuer is not in trusted certificate
-	 * authorities.
-	 */
-	if (issuer == NULL) {
-		out = GNUTLS_CERT_SIGNER_NOT_FOUND | GNUTLS_CERT_INVALID;
-		if (output)
-			*output |= out;
-		gnutls_assert();
-		result = 0;
-		goto cleanup;
-	}
-
 	if (_issuer != NULL)
 		*_issuer = issuer;
 
-	if (nc != NULL) {
-		/* append the issuer's constraints */
-		ret = gnutls_x509_crt_get_name_constraints(issuer, nc, 
-			GNUTLS_NAME_CONSTRAINTS_FLAG_APPEND, NULL);
-		if (ret < 0 && ret != GNUTLS_E_REQUESTED_DATA_NOT_AVAILABLE) {
- nc_fail:
-			out =
-			    GNUTLS_CERT_SIGNER_CONSTRAINTS_FAILURE |
-			    GNUTLS_CERT_INVALID;
-			if (output)
-				*output |= out;
-			gnutls_assert();
-			result = 0;
-			goto cleanup;
-		}
-
-		/* only check name constraints in server certificates, not CAs */
-		if (end_cert != 0) {
-			ret = gnutls_x509_name_constraints_check_crt(nc, GNUTLS_SAN_DNSNAME, cert);
-			if (ret == 0) {
-				gnutls_assert();
-				goto nc_fail;
-			}
-
-			ret = gnutls_x509_name_constraints_check_crt(nc, GNUTLS_SAN_RFC822NAME, cert);
-			if (ret == 0) {
-				gnutls_assert();
-				goto nc_fail;
-			}
-
-			ret = gnutls_x509_name_constraints_check_crt(nc, GNUTLS_SAN_DN, cert);
-			if (ret == 0) {
-				gnutls_assert();
-				goto nc_fail;
-			}
-
-			ret = gnutls_x509_name_constraints_check_crt(nc, GNUTLS_SAN_URI, cert);
-			if (ret == 0) {
-				gnutls_assert();
-				goto nc_fail;
-			}
-
-			ret = gnutls_x509_name_constraints_check_crt(nc, GNUTLS_SAN_IPADDRESS, cert);
-			if (ret == 0) {
-				gnutls_assert();
-				goto nc_fail;
-			}
-		}
-	}
-
-	issuer_version = gnutls_x509_crt_get_version(issuer);
-	if (issuer_version < 0) {
-		gnutls_assert();
-		result = 0;
-		goto cleanup;
-	}
-
-	if (!(flags & GNUTLS_VERIFY_DISABLE_CA_SIGN) &&
-	    ((flags & GNUTLS_VERIFY_DO_NOT_ALLOW_X509_V1_CA_CRT)
-	     || issuer_version != 1)) {
-		if (check_if_ca(cert, issuer, max_path, flags) != 1) {
-			gnutls_assert();
-			out =
-			    GNUTLS_CERT_SIGNER_NOT_CA |
-			    GNUTLS_CERT_INVALID;
-			if (output)
-				*output |= out;
-			result = 0;
-			goto cleanup;
-		}
-
-		ret =
-		    gnutls_x509_crt_get_key_usage(issuer, &usage, NULL);
-		if (ret >= 0) {
-			if (!(usage & GNUTLS_KEY_KEY_CERT_SIGN)) {
-				gnutls_assert();
-				out =
-				    GNUTLS_CERT_SIGNER_CONSTRAINTS_FAILURE
-				    | GNUTLS_CERT_INVALID;
-				if (output)
-					*output |= out;
-				result = 0;
-				goto cleanup;
-			}
-		}
-	}
-
 	ret =
 	    _gnutls_x509_get_signed_data(cert->cert, &cert->der, "tbsCertificate",
 					 &cert_signed_data);
 	if (ret < 0) {
 		result = 0;
 		gnutls_assert();
-		goto cleanup;
+		cert_signed_data.data = NULL;
 	}
 
 	ret =
@@ -712,7 +580,7 @@ verify_crt(gnutls_x509_crt_t cert,
 	if (ret < 0) {
 		result = 0;
 		gnutls_assert();
-		goto cleanup;
+		cert_signature.data = NULL;
 	}
 
 	ret =
@@ -721,75 +589,163 @@ verify_crt(gnutls_x509_crt_t cert,
 	if (ret < 0) {
 		result = 0;
 		gnutls_assert();
-		goto cleanup;
 	}
 	sigalg = ret;
 
-	if (is_level_acceptable(cert, issuer, sigalg, flags) == 0) {
-		gnutls_assert();
-		out =
-		    GNUTLS_CERT_INSECURE_ALGORITHM |
-		    GNUTLS_CERT_INVALID;
-		if (output)
-			*output |= out;
-		result = 0;
-		goto cleanup;
-	}
-
-	/* If the certificate is not self signed check if the algorithms
-	 * used are secure. If the certificate is self signed it doesn't
-	 * really matter.
+	/* issuer is not in trusted certificate
+	 * authorities.
 	 */
-	if (gnutls_sign_is_secure(sigalg) == 0 &&
-	    is_broken_allowed(sigalg, flags) == 0 &&
-	    is_issuer(cert, cert) == 0) {
+	if (issuer == NULL) {
+		out |= GNUTLS_CERT_SIGNER_NOT_FOUND | GNUTLS_CERT_INVALID;
 		gnutls_assert();
-		out =
-		    GNUTLS_CERT_INSECURE_ALGORITHM |
-		    GNUTLS_CERT_INVALID;
-		if (output)
-			*output |= out;
 		result = 0;
-		goto cleanup;
+	} else {
+		if (nc != NULL) {
+			/* append the issuer's constraints */
+			ret = gnutls_x509_crt_get_name_constraints(issuer, nc, 
+				GNUTLS_NAME_CONSTRAINTS_FLAG_APPEND, NULL);
+			if (ret < 0 && ret != GNUTLS_E_REQUESTED_DATA_NOT_AVAILABLE) {
+ nc_fail:
+				out |=
+				    GNUTLS_CERT_SIGNER_CONSTRAINTS_FAILURE |
+				    GNUTLS_CERT_INVALID;
+				gnutls_assert();
+				result = 0;
+				goto nc_done;
+			}
+
+			/* only check name constraints in server certificates, not CAs */
+			if (end_cert != 0) {
+				ret = gnutls_x509_name_constraints_check_crt(nc, GNUTLS_SAN_DNSNAME, cert);
+				if (ret == 0) {
+					gnutls_assert();
+					goto nc_fail;
+				}
+
+				ret = gnutls_x509_name_constraints_check_crt(nc, GNUTLS_SAN_RFC822NAME, cert);
+				if (ret == 0) {
+					gnutls_assert();
+					goto nc_fail;
+				}
+
+				ret = gnutls_x509_name_constraints_check_crt(nc, GNUTLS_SAN_DN, cert);
+				if (ret == 0) {
+					gnutls_assert();
+					goto nc_fail;
+				}
+
+				ret = gnutls_x509_name_constraints_check_crt(nc, GNUTLS_SAN_URI, cert);
+				if (ret == 0) {
+					gnutls_assert();
+					goto nc_fail;
+				}
+
+				ret = gnutls_x509_name_constraints_check_crt(nc, GNUTLS_SAN_IPADDRESS, cert);
+				if (ret == 0) {
+					gnutls_assert();
+					goto nc_fail;
+				}
+			}
+		}
+ nc_done:
+
+		issuer_version = gnutls_x509_crt_get_version(issuer);
+
+		if (issuer_version < 0) {
+			gnutls_assert();
+			result = 0;
+		} else if (!(flags & GNUTLS_VERIFY_DISABLE_CA_SIGN) &&
+	                   ((flags & GNUTLS_VERIFY_DO_NOT_ALLOW_X509_V1_CA_CRT)
+	                    || issuer_version != 1)) {
+			if (check_if_ca(cert, issuer, max_path, flags) != 1) {
+				gnutls_assert();
+				out |=
+				    GNUTLS_CERT_SIGNER_NOT_CA |
+				    GNUTLS_CERT_INVALID;
+				result = 0;
+			}
+
+			ret =
+			    gnutls_x509_crt_get_key_usage(issuer, &usage, NULL);
+			if (ret != GNUTLS_E_REQUESTED_DATA_NOT_AVAILABLE) {
+				if (ret < 0) {
+					gnutls_assert();
+					out |= GNUTLS_CERT_INVALID;
+				} else if (!(usage & GNUTLS_KEY_KEY_CERT_SIGN)) {
+					gnutls_assert();
+					out |=
+					    GNUTLS_CERT_SIGNER_CONSTRAINTS_FAILURE
+					    | GNUTLS_CERT_INVALID;
+					result = 0;
+				}
+			}
+		}
+
+		if (sigalg >= 0) {
+			hash_algo = gnutls_sign_get_hash_algorithm(sigalg);
+			me = mac_to_entry(hash_algo);
+		} else {
+			me = NULL;
+		}
+
+		if (me == NULL) {
+			gnutls_assert();
+			result = 0;
+		} else if (cert_signed_data.data != NULL &&
+		           cert_signature.data != NULL) {
+			ret =
+			    _gnutls_x509_verify_data(me,
+						     &cert_signed_data,
+		        	                     &cert_signature,
+						     issuer);
+			if (ret == GNUTLS_E_PK_SIG_VERIFY_FAILED) {
+				gnutls_assert();
+				out |=
+				    GNUTLS_CERT_INVALID |
+				    GNUTLS_CERT_SIGNATURE_FAILURE;
+				/* error. ignore it */
+				result = 0;
+			} else if (ret < 0) {
+				result = 0;
+				gnutls_assert();
+			}
+		}
 	}
 
-	hash_algo = gnutls_sign_get_hash_algorithm(sigalg);
-	me = mac_to_entry(hash_algo);
-	if (me == NULL) {
-		gnutls_assert();
-		result = 0;
-		goto cleanup;
-	}
+	if (sigalg >= 0) {
+		if (is_level_acceptable(cert, issuer, sigalg, flags) == 0) {
+			gnutls_assert();
+			out |=
+			    GNUTLS_CERT_INSECURE_ALGORITHM |
+			    GNUTLS_CERT_INVALID;
+			result = 0;
+		}
 
-	ret =
-	    _gnutls_x509_verify_data(me,
-				     &cert_signed_data, &cert_signature,
-				     issuer);
-	if (ret == GNUTLS_E_PK_SIG_VERIFY_FAILED) {
-		gnutls_assert();
-		out |= GNUTLS_CERT_INVALID | GNUTLS_CERT_SIGNATURE_FAILURE;
-		/* error. ignore it */
-		if (output)
-			*output |= out;
-		result = 0;
-	} else if (ret < 0) {
-		result = 0;
-		gnutls_assert();
-		goto cleanup;
-	} else if (ret == 1)
-		result = 1;
+		/* If the certificate is not self signed check if the algorithms
+		 * used are secure. If the certificate is self signed it doesn't
+		 * really matter.
+		 */
+		if (gnutls_sign_is_secure(sigalg) == 0 &&
+		    is_broken_allowed(sigalg, flags) == 0 &&
+		    is_issuer(cert, cert) == 0) {
+			gnutls_assert();
+			out |=
+			    GNUTLS_CERT_INSECURE_ALGORITHM |
+			    GNUTLS_CERT_INVALID;
+			result = 0;
+		}
+	}
 
 	/* Check activation/expiration times
 	 */
 	if (!(flags & GNUTLS_VERIFY_DISABLE_TIME_CHECKS)) {
 		/* check the time of the issuer first */
-		if (!(flags & GNUTLS_VERIFY_DISABLE_TRUSTED_TIME_CHECKS)) {
+		if (issuer != NULL &&
+		    !(flags & GNUTLS_VERIFY_DISABLE_TRUSTED_TIME_CHECKS)) {
 			out |= check_time_status(issuer, now);
 			if (out != 0) {
 				gnutls_assert();
 				result = 0;
-				if (output)
-					*output |= out;
 			}
 		}
 
@@ -797,12 +753,13 @@ verify_crt(gnutls_x509_crt_t cert,
 		if (out != 0) {
 			gnutls_assert();
 			result = 0;
-			if (output)
-				*output |= out;
 		}
 	}
 
       cleanup:
+	if (output)
+		*output |= out;
+
 	if (func) {
 		if (result == 0) {
 			out |= GNUTLS_CERT_INVALID;
@@ -850,6 +807,7 @@ _gnutls_verify_crt_status(const gnutls_x509_crt_t * certificate_list,
 				const gnutls_x509_crt_t * trusted_cas,
 				int tcas_size,
 				unsigned int flags,
+				const char *purpose,
 				gnutls_verify_output_function func)
 {
 	int i = 0, ret;
@@ -966,6 +924,20 @@ _gnutls_verify_crt_status(const gnutls_x509_crt_t * certificate_list,
 		if (i - 1 < 0)
 			break;
 
+		if (purpose != NULL) {
+			ret = _gnutls_check_key_purpose(certificate_list[i], purpose, 1);
+			if (ret != 1) {
+				gnutls_assert();
+				status |= GNUTLS_CERT_INVALID;
+				status |= GNUTLS_CERT_PURPOSE_MISMATCH;
+
+				if (func)
+					func(certificate_list[i-1],
+					     certificate_list[i], NULL, status);
+				goto cleanup;
+			}
+		}
+
 		/* note that here we disable this V1 CA flag. So that no version 1
 		 * certificates can exist in a supplied chain.
 		 */
@@ -985,7 +957,6 @@ _gnutls_verify_crt_status(const gnutls_x509_crt_t * certificate_list,
 		}
 	}
 
-	status = 0;
 cleanup:
 	gnutls_x509_name_constraints_deinit(nc);
 	return status;
@@ -993,16 +964,17 @@ cleanup:
 
 /* Returns true if the provided purpose is in accordance with the certificate.
  */
-bool _gnutls_check_key_purpose(gnutls_x509_crt_t cert, const char *purpose)
+bool _gnutls_check_key_purpose(gnutls_x509_crt_t cert, const char *purpose, unsigned no_any)
 {
 	char oid[MAX_OID_SIZE];
 	size_t oid_size;
 	int ret;
+	unsigned critical = 0;
 	unsigned i;
 
 	for (i=0;;i++) {
 		oid_size = sizeof(oid);
-		ret = gnutls_x509_crt_get_key_purpose_oid(cert, i, oid, &oid_size, NULL);
+		ret = gnutls_x509_crt_get_key_purpose_oid(cert, i, oid, &oid_size, &critical);
 		if (ret == GNUTLS_E_REQUESTED_DATA_NOT_AVAILABLE) {
 			if (i==0) {
 				/* no key purpose in certificate, assume ANY */
@@ -1016,7 +988,7 @@ bool _gnutls_check_key_purpose(gnutls_x509_crt_t cert, const char *purpose)
 			break;
 		}
 
-		if (strcmp(oid, purpose) == 0 || strcmp(oid, GNUTLS_KP_ANY) == 0) {
+		if (strcmp(oid, purpose) == 0 || (no_any == 0 && strcmp(oid, GNUTLS_KP_ANY) == 0)) {
 			return 1;
 		}
 	}
@@ -1141,12 +1113,18 @@ _gnutls_pkcs11_verify_crt_status(const char* url,
 			 	GNUTLS_PKCS11_OBJ_FLAG_RETRIEVE_TRUSTED|GNUTLS_PKCS11_OBJ_FLAG_COMPARE);
 			if (ret != 0) {
 				return _gnutls_verify_crt_status(certificate_list, clist_size,
-					&certificate_list[clist_size - 1], 1, flags, func);
+					&certificate_list[clist_size - 1], 1, flags,
+					purpose, func);
 			}
 		}
 
 		status |= GNUTLS_CERT_INVALID;
 		status |= GNUTLS_CERT_SIGNER_NOT_FOUND;
+		/* verify the certificate list against 0 trusted CAs in order
+		 * to get, any additional flags from the certificate list (e.g.,
+		 * insecure algorithms or expired */
+		status |= _gnutls_verify_crt_status(certificate_list, clist_size,
+						    NULL, 0, flags, purpose, func);
 		goto cleanup;
 	}
 
@@ -1166,18 +1144,19 @@ _gnutls_pkcs11_verify_crt_status(const char* url,
 		goto cleanup;
 	}
 
-	if (purpose != NULL) {
-		ret = _gnutls_check_key_purpose(issuer, purpose);
-		if (ret != 1) {
-			gnutls_assert();
-			status |= GNUTLS_CERT_INVALID;
-			status |= GNUTLS_CERT_SIGNER_NOT_FOUND;
-			goto cleanup;
-		}
+	/* security modules that provide trust, bundle all certificates (of all purposes)
+	 * together. In software that doesn't specify any purpose assume the default to
+	 * be www-server. */
+	ret = _gnutls_check_key_purpose(issuer, purpose==NULL?GNUTLS_KP_TLS_WWW_SERVER:purpose, 0);
+	if (ret != 1) {
+		gnutls_assert();
+		status |= GNUTLS_CERT_INVALID;
+		status |= GNUTLS_CERT_SIGNER_NOT_FOUND;
+		goto cleanup;
 	}
 
 	status = _gnutls_verify_crt_status(certificate_list, clist_size,
-				&issuer, 1, flags, func);
+				&issuer, 1, flags, purpose, func);
 
 cleanup:
 	gnutls_free(raw_issuer.data);
@@ -1188,20 +1167,6 @@ cleanup:
 }
 #endif
 
-/* This will return the appropriate hash to verify the given signature.
- * If signature is NULL it will return an (or the) appropriate hash for
- * the given parameters.
- */
-int
-_gnutls_x509_verify_algorithm(gnutls_digest_algorithm_t * hash,
-			      const gnutls_datum_t * signature,
-			      gnutls_pk_algorithm_t pk,
-			      gnutls_pk_params_st * issuer_params)
-{
-	return _gnutls_pk_hash_algorithm(pk, signature, issuer_params,
-					 hash);
-}
-
 /* verifies if the certificate is properly signed.
  * returns GNUTLS_E_PK_VERIFY_SIG_FAILED on failure and 1 on success.
  * 
@@ -1251,13 +1216,12 @@ _gnutls_x509_verify_data(const mac_entry_st * me,
  * @flags: Flags that may be used to change the verification algorithm. Use OR of the gnutls_certificate_verify_flags enumerations.
  * @verify: will hold the certificate verification output.
  *
- * This function will try to verify the given certificate list and
- * return its status.  If no flags are specified (0), this function
- * will use the basicConstraints (2.5.29.19) PKIX extension. This
- * means that only a certificate authority is allowed to sign a
- * certificate.
  *
- * You must also check the peer's name in order to check if the verified
+ * This function will try to verify the given certificate list and
+ * return its status. The details of the verification are the same
+ * as in gnutls_x509_trust_list_verify_crt2().
+ *
+ * You must check the peer's name in order to check if the verified
  * certificate belongs to the actual peer.
  *
  * The certificate verification output will be put in @verify and will
@@ -1287,7 +1251,7 @@ gnutls_x509_crt_list_verify(const gnutls_x509_crt_t * cert_list,
 	*verify =
 	    _gnutls_verify_crt_status(cert_list, cert_list_length,
 					    CA_list, CA_list_length,
-					    flags, NULL);
+					    flags, NULL, NULL);
 
 	/* Check for revoked certificates in the chain. 
 	 */
@@ -1316,6 +1280,9 @@ gnutls_x509_crt_list_verify(const gnutls_x509_crt_t * cert_list,
  * its status. Note that a verification error does not imply a negative
  * return status. In that case the @verify status is set.
  *
+ * The details of the verification are the same
+ * as in gnutls_x509_trust_list_verify_crt2().
+ *
  * Returns: On success, %GNUTLS_E_SUCCESS (0) is returned, otherwise a
  *   negative error value.
  **/
@@ -1330,7 +1297,7 @@ gnutls_x509_crt_verify(gnutls_x509_crt_t cert,
 	*verify =
 	    _gnutls_verify_crt_status(&cert, 1,
 					    CA_list, CA_list_length,
-					    flags, NULL);
+					    flags, NULL, NULL);
 	return 0;
 }
 
@@ -1410,42 +1377,6 @@ gnutls_x509_crl_verify(gnutls_x509_crl_t crl,
 	if (tcas_size >= 1)
 		issuer = find_crl_issuer(crl, trusted_cas, tcas_size);
 
-	/* issuer is not in trusted certificate
-	 * authorities.
-	 */
-	if (issuer == NULL) {
-		gnutls_assert();
-		if (verify)
-			*verify |=
-			    GNUTLS_CERT_SIGNER_NOT_FOUND |
-			    GNUTLS_CERT_INVALID;
-		return 0;
-	}
-
-	if (!(flags & GNUTLS_VERIFY_DISABLE_CA_SIGN)) {
-		if (gnutls_x509_crt_get_ca_status(issuer, NULL) != 1) {
-			gnutls_assert();
-			if (verify)
-				*verify |=
-				    GNUTLS_CERT_SIGNER_NOT_CA |
-				    GNUTLS_CERT_INVALID;
-			return 0;
-		}
-
-		result =
-		    gnutls_x509_crt_get_key_usage(issuer, &usage, NULL);
-		if (result >= 0) {
-			if (!(usage & GNUTLS_KEY_CRL_SIGN)) {
-				gnutls_assert();
-				if (verify)
-					*verify |=
-					    GNUTLS_CERT_SIGNER_CONSTRAINTS_FAILURE
-					    | GNUTLS_CERT_INVALID;
-				return 0;
-			}
-		}
-	}
-
 	result =
 	    _gnutls_x509_get_signed_data(crl->crl, &crl->der, "tbsCertList",
 					 &crl_signed_data);
@@ -1478,21 +1409,58 @@ gnutls_x509_crl_verify(gnutls_x509_crl_t crl,
 
 	hash_algo = gnutls_sign_get_hash_algorithm(result);
 
-	result =
-	    _gnutls_x509_verify_data(mac_to_entry(hash_algo),
-				     &crl_signed_data, &crl_signature,
-				     issuer);
-	if (result == GNUTLS_E_PK_SIG_VERIFY_FAILED) {
-		gnutls_assert();
-		/* error. ignore it */
-		if (verify)
-			*verify |= GNUTLS_CERT_SIGNATURE_FAILURE;
-		result = 0;
-	} else if (result < 0) {
+	/* issuer is not in trusted certificate
+	 * authorities.
+	 */
+	if (issuer == NULL) {
 		gnutls_assert();
 		if (verify)
-			*verify |= GNUTLS_CERT_INVALID;
-		goto cleanup;
+			*verify |=
+			    GNUTLS_CERT_SIGNER_NOT_FOUND |
+			    GNUTLS_CERT_INVALID;
+	} else {
+		if (!(flags & GNUTLS_VERIFY_DISABLE_CA_SIGN)) {
+			if (gnutls_x509_crt_get_ca_status(issuer, NULL) != 1) {
+				gnutls_assert();
+				if (verify)
+					*verify |=
+					    GNUTLS_CERT_SIGNER_NOT_CA |
+					    GNUTLS_CERT_INVALID;
+			}
+
+			result =
+			    gnutls_x509_crt_get_key_usage(issuer, &usage, NULL);
+			if (result != GNUTLS_E_REQUESTED_DATA_NOT_AVAILABLE) {
+				if (result < 0) {
+					gnutls_assert();
+					if (verify)
+						*verify |= GNUTLS_CERT_INVALID;
+				} else if (!(usage & GNUTLS_KEY_CRL_SIGN)) {
+					gnutls_assert();
+					if (verify)
+						*verify |=
+						    GNUTLS_CERT_SIGNER_CONSTRAINTS_FAILURE
+						    | GNUTLS_CERT_INVALID;
+				}
+			}
+		}
+
+		result =
+		    _gnutls_x509_verify_data(mac_to_entry(hash_algo),
+					     &crl_signed_data, &crl_signature,
+					     issuer);
+		if (result == GNUTLS_E_PK_SIG_VERIFY_FAILED) {
+			gnutls_assert();
+			/* error. ignore it */
+			if (verify)
+				*verify |= GNUTLS_CERT_SIGNATURE_FAILURE;
+			result = 0;
+		} else if (result < 0) {
+			gnutls_assert();
+			if (verify)
+				*verify |= GNUTLS_CERT_INVALID;
+			goto cleanup;
+		}
 	}
 
 	{
