@@ -1,5 +1,6 @@
 /*
- * Copyright (C) 2014 Free Software Foundation
+ * Copyright (C) 2014-2016 Free Software Foundation, Inc.
+ * Copyright (C) 2016 Red Hat, Inc.
  *
  * This file is part of GnuTLS.
  *
@@ -21,22 +22,17 @@
 /* This file contains functions to handle X.509 certificate extensions (the x509-ext API)
  */
 
-#include <gnutls_int.h>
-
-#include <gnutls_datum.h>
-#include <gnutls_errors.h>
+#include "gnutls_int.h"
+#include <datum.h>
+#include "errors.h"
 #include <common.h>
-#include <gnutls_x509.h>
+#include <x509.h>
 #include <x509_b64.h>
 #include <c-ctype.h>
+#include "x509_ext_int.h"
+#include "virt-san.h"
 #include <gnutls/x509-ext.h>
 
-struct name_st {
-	unsigned int type;
-	gnutls_datum_t san;
-	gnutls_datum_t othername_oid;
-};
-
 #define MAX_ENTRIES 64
 struct gnutls_subject_alt_names_st {
 	struct name_st *names;
@@ -130,14 +126,16 @@ int gnutls_subject_alt_names_get(gnutls_subject_alt_names_t sans,
 }
 
 /* This is the same as gnutls_subject_alt_names_set() but will not
- * copy the strings */
+ * copy the strings. It expects all the provided input to be already
+ * allocated by gnutls. */
 static
 int subject_alt_names_set(struct name_st **names,
 			  unsigned int *size,
 			  unsigned int san_type,
-			  const gnutls_datum_t * san, char *othername_oid)
+			  gnutls_datum_t * san, char *othername_oid)
 {
 	void *tmp;
+	int ret;
 
 	tmp = gnutls_realloc(*names, (*size + 1) * sizeof((*names)[0]));
 	if (tmp == NULL) {
@@ -145,17 +143,9 @@ int subject_alt_names_set(struct name_st **names,
 	}
 	*names = tmp;
 
-	(*names)[*size].type = san_type;
-	(*names)[*size].san.data = san->data;
-	(*names)[*size].san.size = san->size;
-
-	if (othername_oid) {
-		(*names)[*size].othername_oid.data = (uint8_t *) othername_oid;
-		(*names)[*size].othername_oid.size = strlen(othername_oid);
-	} else {
-		(*names)[*size].othername_oid.data = NULL;
-		(*names)[*size].othername_oid.size = 0;
-	}
+	ret = _gnutls_alt_name_assign_virt_type(&(*names)[*size], san_type, san, othername_oid);
+	if (ret < 0)
+		return gnutls_assert_val(ret);
 
 	(*size)++;
 	return 0;
@@ -318,13 +308,15 @@ int gnutls_x509_ext_export_subject_alt_names(gnutls_subject_alt_names_t sans,
 
 	for (i = 0; i < sans->size; i++) {
 		if (sans->names[i].type == GNUTLS_SAN_OTHERNAME) {
-			ret = gnutls_assert_val(GNUTLS_E_UNIMPLEMENTED_FEATURE);
-			goto cleanup;
+			ret = _gnutls_write_new_othername(c2, "", (char*)sans->names[i].othername_oid.data,
+					      sans->names[i].san.data, sans->names[i].san.size);
+		} else {
+			ret =
+			    _gnutls_write_new_general_name(c2, "", sans->names[i].type,
+							   sans->names[i].san.data,
+							   sans->names[i].san.size);
 		}
-		ret =
-		    _gnutls_write_new_general_name(c2, "", sans->names[i].type,
-						   sans->names[i].san.data,
-						   sans->names[i].san.size);
+
 		if (ret < 0) {
 			gnutls_assert();
 			goto cleanup;
@@ -357,8 +349,10 @@ int gnutls_x509_ext_export_subject_alt_names(gnutls_subject_alt_names_t sans,
  *
  * When the @flags is set to %GNUTLS_NAME_CONSTRAINTS_FLAG_APPEND, then if 
  * the @nc type is empty this function will behave identically as if the flag was not set.
- * Otherwise if there are elements in the @nc type then only the
- * excluded constraints will be appended to the constraints.
+ * Otherwise if there are elements in the @nc structure then the
+ * constraints will be merged with the existing constraints following
+ * RFC5280 p6.1.4 (excluded constraints will be appended, permitted
+ * will be intersected).
  *
  * Note that @nc must be initialized prior to calling this function.
  *
@@ -373,6 +367,7 @@ int gnutls_x509_ext_import_name_constraints(const gnutls_datum_t * ext,
 {
 	int result, ret;
 	ASN1_TYPE c2 = ASN1_TYPE_EMPTY;
+	gnutls_x509_name_constraints_t nc2 = NULL;
 
 	result = asn1_create_element
 	    (_gnutls_get_pkix(), "PKIX1.NameConstraints", &c2);
@@ -388,8 +383,39 @@ int gnutls_x509_ext_import_name_constraints(const gnutls_datum_t * ext,
 		goto cleanup;
 	}
 
-	if (!(flags & GNUTLS_NAME_CONSTRAINTS_FLAG_APPEND)
-	    || (nc->permitted == NULL && nc->excluded == NULL)) {
+	if (flags & GNUTLS_NAME_CONSTRAINTS_FLAG_APPEND &&
+	    (nc->permitted != NULL || nc->excluded != NULL)) {
+		ret = gnutls_x509_name_constraints_init (&nc2);
+		if (ret < 0) {
+			gnutls_assert();
+			goto cleanup;
+		}
+
+		ret =
+		    _gnutls_extract_name_constraints(c2, "permittedSubtrees",
+						     &nc2->permitted);
+		if (ret < 0) {
+			gnutls_assert();
+			goto cleanup;
+		}
+
+		ret =
+		    _gnutls_extract_name_constraints(c2, "excludedSubtrees",
+						     &nc2->excluded);
+		if (ret < 0) {
+			gnutls_assert();
+			goto cleanup;
+		}
+
+		ret = _gnutls_x509_name_constraints_merge(nc, nc2);
+		if (ret < 0) {
+			gnutls_assert();
+			goto cleanup;
+		}
+	} else {
+		_gnutls_name_constraints_node_free(nc->permitted);
+		_gnutls_name_constraints_node_free(nc->excluded);
+
 		ret =
 		    _gnutls_extract_name_constraints(c2, "permittedSubtrees",
 						     &nc->permitted);
@@ -397,20 +423,22 @@ int gnutls_x509_ext_import_name_constraints(const gnutls_datum_t * ext,
 			gnutls_assert();
 			goto cleanup;
 		}
-	}
 
-	ret =
-	    _gnutls_extract_name_constraints(c2, "excludedSubtrees",
-					     &nc->excluded);
-	if (ret < 0) {
-		gnutls_assert();
-		goto cleanup;
+		ret =
+		    _gnutls_extract_name_constraints(c2, "excludedSubtrees",
+						     &nc->excluded);
+		if (ret < 0) {
+			gnutls_assert();
+			goto cleanup;
+		}
 	}
 
 	ret = 0;
 
  cleanup:
 	asn1_delete_structure(&c2);
+	if (nc2)
+		gnutls_x509_name_constraints_deinit (nc2);
 
 	return ret;
 }
@@ -1486,8 +1514,9 @@ int gnutls_x509_ext_import_proxy(const gnutls_datum_t * ext, int *pathlen,
 
 /**
  * gnutls_x509_ext_export_proxy:
- * @pathLenConstraint: A negative value will remove the path length constraint,
- *   while non-negative values will be set as the length of the pathLenConstraints field.
+ * @pathLenConstraint: non-negative error codes indicate maximum length of path,
+ *   and negative error codes indicate that the pathLenConstraints field should
+ *   not be present.
  * @policyLanguage: OID describing the language of @policy.
  * @policy: uint8_t byte array with policy language, can be %NULL
  * @sizeof_policy: size of @policy.
@@ -1495,7 +1524,8 @@ int gnutls_x509_ext_import_proxy(const gnutls_datum_t * ext, int *pathlen,
  *
  * This function will convert the parameters provided to a proxyCertInfo extension.
  *
- * The @ext data will be allocated using gnutls_malloc().
+ * The @ext data will be allocated using
+ * gnutls_malloc().
  *
  * Returns: On success, %GNUTLS_E_SUCCESS (0) is returned, otherwise a
  *   negative error value.
@@ -1865,7 +1895,7 @@ int gnutls_x509_ext_import_policies(const gnutls_datum_t * ext,
 				policies->policy[j].qualifier[i].type =
 				    GNUTLS_X509_QUALIFIER_URI;
 			} else if (strcmp(tmpoid, "1.3.6.1.5.5.7.2.2") == 0) {
-				gnutls_datum_t txt;
+				gnutls_datum_t txt = {NULL, 0};
 
 				snprintf(tmpstr, sizeof(tmpstr),
 					 "?%u.policyQualifiers.?%u.qualifier",
@@ -3121,46 +3151,3 @@ int _gnutls_x509_decode_ext(const gnutls_datum_t *der, gnutls_x509_ext_st *out)
 	
 }
 
-/**
- * gnutls_x509_othername_to_virtual:
- * @oid: The othername object identifier
- * @othername: The othername data
- * @virt_type: GNUTLS_SAN_OTHERNAME_XXX
- * @virt: allocated printable data
- *
- * This function will parse and convert the othername data to a virtual
- * type supported by gnutls.
- *
- * Returns: On success, %GNUTLS_E_SUCCESS (0) is returned, otherwise a negative error value.
- *
- * Since: 3.3.8
- **/
-int gnutls_x509_othername_to_virtual(const char *oid, 
-				     const gnutls_datum_t *othername,
-				     unsigned int *virt_type,
-				     gnutls_datum_t *virt)
-{
-	int ret;
-	unsigned type;
-
-	type = _san_othername_to_virtual(oid, strlen(oid));
-	if (type == GNUTLS_SAN_OTHERNAME)
-		return gnutls_assert_val(GNUTLS_E_X509_UNKNOWN_SAN);
-
-	if (virt_type)
-		*virt_type = type;
-
-	switch(type) {
-		case GNUTLS_SAN_OTHERNAME_XMPP:
-			ret = _gnutls_x509_decode_string
-				    (ASN1_ETYPE_UTF8_STRING, othername->data,
-				     othername->size, virt, 0);
-			if (ret < 0) {
-				gnutls_assert();
-				return ret;
-			}
-			return 0;
-		default:
-			return gnutls_assert_val(GNUTLS_E_INTERNAL_ERROR);
-	}
-}
