@@ -23,13 +23,13 @@
 /* This file contains functions to handle X.509 certificate generation.
  */
 
-#include <gnutls_int.h>
+#include "gnutls_int.h"
 
-#include <gnutls_datum.h>
-#include <gnutls_global.h>
-#include <gnutls_errors.h>
+#include <datum.h>
+#include <global.h>
+#include "errors.h"
 #include <common.h>
-#include <gnutls_x509.h>
+#include <x509.h>
 #include <gnutls/x509-ext.h>
 #include <x509_b64.h>
 #include "x509_int.h"
@@ -67,6 +67,8 @@ gnutls_x509_crt_set_dn_by_oid(gnutls_x509_crt_t crt, const char *oid,
 		return GNUTLS_E_INVALID_REQUEST;
 	}
 
+	MODIFIED(crt);
+
 	return _gnutls_x509_set_dn_oid(crt->cert, "tbsCertificate.subject",
 				       oid, raw_flag, name, sizeof_name);
 }
@@ -107,6 +109,8 @@ gnutls_x509_crt_set_issuer_dn_by_oid(gnutls_x509_crt_t crt,
 		return GNUTLS_E_INVALID_REQUEST;
 	}
 
+	MODIFIED(crt);
+
 	return _gnutls_x509_set_dn_oid(crt->cert, "tbsCertificate.issuer",
 				       oid, raw_flag, name, sizeof_name);
 }
@@ -140,6 +144,8 @@ gnutls_x509_crt_set_proxy_dn(gnutls_x509_crt_t crt,
 		return GNUTLS_E_INVALID_REQUEST;
 	}
 
+	MODIFIED(crt);
+
 	result = asn1_copy_node(crt->cert, "tbsCertificate.subject",
 				eecrt->cert, "tbsCertificate.subject");
 	if (result != ASN1_SUCCESS) {
@@ -186,6 +192,8 @@ gnutls_x509_crt_set_version(gnutls_x509_crt_t crt, unsigned int version)
 		return GNUTLS_E_INVALID_REQUEST;
 	}
 
+	MODIFIED(crt);
+
 	if (null > 0)
 		null--;
 
@@ -225,6 +233,8 @@ gnutls_x509_crt_set_key(gnutls_x509_crt_t crt, gnutls_x509_privkey_t key)
 		return GNUTLS_E_INVALID_REQUEST;
 	}
 
+	MODIFIED(crt);
+
 	result = _gnutls_x509_encode_and_copy_PKI_params(crt->cert,
 							 "tbsCertificate.subjectPublicKeyInfo",
 							 key->pk_algorithm,
@@ -262,6 +272,8 @@ int gnutls_x509_crt_set_crq(gnutls_x509_crt_t crt, gnutls_x509_crq_t crq)
 		return GNUTLS_E_INVALID_REQUEST;
 	}
 
+	MODIFIED(crt);
+
 	result = gnutls_x509_crq_verify(crq, 0);
 	if (result < 0)
 		return gnutls_assert_val(result);
@@ -310,6 +322,8 @@ gnutls_x509_crt_set_crq_extensions(gnutls_x509_crt_t crt,
 		return GNUTLS_E_INVALID_REQUEST;
 	}
 
+	MODIFIED(crt);
+
 	for (i = 0;; i++) {
 		int result;
 		char oid[MAX_OID_SIZE];
@@ -368,9 +382,6 @@ gnutls_x509_crt_set_crq_extensions(gnutls_x509_crt_t crt,
 		}
 	}
 
-	if (i > 0)
-		crt->use_extensions = 1;
-
 	return 0;
 }
 
@@ -413,8 +424,6 @@ gnutls_x509_crt_set_extension_by_oid(gnutls_x509_crt_t crt,
 		return result;
 	}
 
-	crt->use_extensions = 1;
-
 	return 0;
 
 }
@@ -463,8 +472,6 @@ gnutls_x509_crt_set_basic_constraints(gnutls_x509_crt_t crt,
 		return result;
 	}
 
-	crt->use_extensions = 1;
-
 	return 0;
 }
 
@@ -525,8 +532,6 @@ gnutls_x509_crt_set_key_usage(gnutls_x509_crt_t crt, unsigned int usage)
 		return result;
 	}
 
-	crt->use_extensions = 1;
-
 	return 0;
 }
 
@@ -577,17 +582,10 @@ gnutls_x509_crt_set_subject_alternative_name(gnutls_x509_crt_t crt,
  * @flags: GNUTLS_FSAN_SET to clear previous data or GNUTLS_FSAN_APPEND to append. 
  *
  * This function will set the subject alternative name certificate
- * extension. It can set the following types:
- *
- * %GNUTLS_SAN_DNSNAME: as a text string
- *
- * %GNUTLS_SAN_RFC822NAME: as a text string
- *
- * %GNUTLS_SAN_URI: as a text string
- *
- * %GNUTLS_SAN_IPADDRESS: as a binary IP address (4 or 16 bytes)
- * 
- * Other values can be set as binary values with the proper DER encoding.
+ * extension. It can set the following types: %GNUTLS_SAN_DNSNAME as a text string,
+ * %GNUTLS_SAN_RFC822NAME as a text string, %GNUTLS_SAN_URI as a text string,
+ * %GNUTLS_SAN_IPADDRESS as a binary IP address (4 or 16 bytes),
+ * %GNUTLS_SAN_OTHERNAME_XMPP as a UTF8 string (since 3.5.0).
  *
  * Returns: On success, %GNUTLS_E_SUCCESS (0) is returned, otherwise a
  *   negative error value.
@@ -614,7 +612,7 @@ gnutls_x509_crt_set_subject_alt_name(gnutls_x509_crt_t crt,
 	/* Check if the extension already exists.
 	 */
 
-	if (flags == GNUTLS_FSAN_APPEND) {
+	if (flags & GNUTLS_FSAN_APPEND) {
 		result =
 		    _gnutls_x509_crt_get_extension(crt, "2.5.29.17", 0,
 						   &prev_der_data,
@@ -629,13 +627,10 @@ gnutls_x509_crt_set_subject_alt_name(gnutls_x509_crt_t crt,
 	/* generate the extension.
 	 */
 	result =
-	    _gnutls_x509_ext_gen_subject_alt_name(type, data, data_size,
+	    _gnutls_x509_ext_gen_subject_alt_name(type, NULL, data, data_size,
 						  &prev_der_data,
 						  &der_data);
 
-	if (flags == GNUTLS_FSAN_APPEND)
-		_gnutls_free_datum(&prev_der_data);
-
 	if (result < 0) {
 		gnutls_assert();
 		goto finish;
@@ -652,9 +647,7 @@ gnutls_x509_crt_set_subject_alt_name(gnutls_x509_crt_t crt,
 		return result;
 	}
 
-	crt->use_extensions = 1;
-
-	return 0;
+	result = 0;
 
       finish:
 	_gnutls_free_datum(&prev_der_data);
@@ -697,7 +690,7 @@ gnutls_x509_crt_set_issuer_alt_name(gnutls_x509_crt_t crt,
 	/* Check if the extension already exists.
 	 */
 
-	if (flags == GNUTLS_FSAN_APPEND) {
+	if (flags & GNUTLS_FSAN_APPEND) {
 		result =
 		    _gnutls_x509_crt_get_extension(crt, "2.5.29.18", 0,
 						   &prev_der_data,
@@ -712,13 +705,10 @@ gnutls_x509_crt_set_issuer_alt_name(gnutls_x509_crt_t crt,
 	/* generate the extension.
 	 */
 	result =
-	    _gnutls_x509_ext_gen_subject_alt_name(type, data, data_size,
+	    _gnutls_x509_ext_gen_subject_alt_name(type, NULL, data, data_size,
 						  &prev_der_data,
 						  &der_data);
 
-	if (flags == GNUTLS_FSAN_APPEND)
-		_gnutls_free_datum(&prev_der_data);
-
 	if (result < 0) {
 		gnutls_assert();
 		goto finish;
@@ -735,15 +725,209 @@ gnutls_x509_crt_set_issuer_alt_name(gnutls_x509_crt_t crt,
 		return result;
 	}
 
-	crt->use_extensions = 1;
-
-	return 0;
+	result = 0;
 
       finish:
 	_gnutls_free_datum(&prev_der_data);
 	return result;
 }
 
+int _gnutls_encode_othername_data(unsigned flags, const void *data, unsigned data_size, gnutls_datum_t *output)
+{
+	int ret;
+	if (flags & GNUTLS_FSAN_ENCODE_OCTET_STRING) {
+		ret = _gnutls_x509_encode_string(ASN1_ETYPE_OCTET_STRING,
+			data, data_size, output);
+	} else if (flags & GNUTLS_FSAN_ENCODE_UTF8_STRING) {
+		ret = _gnutls_x509_encode_string(ASN1_ETYPE_UTF8_STRING,
+			data, data_size, output);
+	} else {
+		ret = _gnutls_set_datum(output, data, data_size);
+	}
+	return ret;
+}
+
+/**
+ * gnutls_x509_crt_set_subject_alt_othername:
+ * @crt: a certificate of type #gnutls_x509_crt_t
+ * @oid: The other name OID
+ * @data: The data to be set
+ * @data_size: The size of data to be set
+ * @flags: GNUTLS_FSAN_SET to clear previous data or GNUTLS_FSAN_APPEND to append. 
+ *
+ * This function will set an "othername" to the subject alternative name certificate
+ * extension.
+ *
+ * The values set are set as binary values and are expected to have the proper DER encoding.
+ * For convenience the flags %GNUTLS_FSAN_ENCODE_OCTET_STRING and %GNUTLS_FSAN_ENCODE_UTF8_STRING
+ * can be used to encode the provided data.
+ *
+ * Returns: On success, %GNUTLS_E_SUCCESS (0) is returned, otherwise a
+ *   negative error value.
+ *
+ * Since: 3.5.0
+ **/
+int
+gnutls_x509_crt_set_subject_alt_othername(gnutls_x509_crt_t crt,
+				     const char *oid,
+				     const void *data,
+				     unsigned int data_size,
+				     unsigned int flags)
+{
+	int result;
+	gnutls_datum_t der_data = { NULL, 0 };
+	gnutls_datum_t prev_der_data = { NULL, 0 };
+	gnutls_datum_t encoded_data = { NULL, 0 };
+	unsigned int critical = 0;
+
+	if (crt == NULL) {
+		gnutls_assert();
+		return GNUTLS_E_INVALID_REQUEST;
+	}
+
+	/* Check if the extension already exists.
+	 */
+
+	if (flags & GNUTLS_FSAN_APPEND) {
+		result =
+		    _gnutls_x509_crt_get_extension(crt, "2.5.29.17", 0,
+						   &prev_der_data,
+						   &critical);
+		if (result < 0
+		    && result != GNUTLS_E_REQUESTED_DATA_NOT_AVAILABLE) {
+			gnutls_assert();
+			return result;
+		}
+	}
+
+	result = _gnutls_encode_othername_data(flags, data, data_size, &encoded_data);
+	if (result < 0) {
+		gnutls_assert();
+		goto finish;
+	}
+
+	/* generate the extension.
+	 */
+	result =
+	    _gnutls_x509_ext_gen_subject_alt_name(GNUTLS_SAN_OTHERNAME, oid,
+	    					  encoded_data.data, encoded_data.size,
+						  &prev_der_data,
+						  &der_data);
+
+	if (result < 0) {
+		gnutls_assert();
+		goto finish;
+	}
+
+	result =
+	    _gnutls_x509_crt_set_extension(crt, "2.5.29.17", &der_data,
+					   critical);
+
+
+	if (result < 0) {
+		gnutls_assert();
+		goto finish;
+	}
+
+	result = 0;
+
+      finish:
+	_gnutls_free_datum(&der_data);
+	_gnutls_free_datum(&prev_der_data);
+	_gnutls_free_datum(&encoded_data);
+	return result;
+}
+
+/**
+ * gnutls_x509_crt_set_issuer_alt_othername:
+ * @crt: a certificate of type #gnutls_x509_crt_t
+ * @oid: The other name OID
+ * @data: The data to be set
+ * @data_size: The size of data to be set
+ * @flags: GNUTLS_FSAN_SET to clear previous data or GNUTLS_FSAN_APPEND to append. 
+ *
+ * This function will set an "othername" to the issuer alternative name certificate
+ * extension.
+ *
+ * The values set are set as binary values and are expected to have the proper DER encoding.
+ * For convenience the flags %GNUTLS_FSAN_ENCODE_OCTET_STRING and %GNUTLS_FSAN_ENCODE_UTF8_STRING
+ * can be used to encode the provided data.
+ *
+ * Returns: On success, %GNUTLS_E_SUCCESS (0) is returned, otherwise a
+ *   negative error value.
+ *
+ * Since: 3.5.0
+ **/
+int
+gnutls_x509_crt_set_issuer_alt_othername(gnutls_x509_crt_t crt,
+				     const char *oid,
+				     const void *data,
+				     unsigned int data_size,
+				     unsigned int flags)
+{
+	int result;
+	gnutls_datum_t der_data = { NULL, 0 };
+	gnutls_datum_t prev_der_data = { NULL, 0 };
+	gnutls_datum_t encoded_data = {NULL, 0};
+	unsigned int critical = 0;
+
+	if (crt == NULL) {
+		gnutls_assert();
+		return GNUTLS_E_INVALID_REQUEST;
+	}
+
+	/* Check if the extension already exists.
+	 */
+
+	if (flags & GNUTLS_FSAN_APPEND) {
+		result =
+		    _gnutls_x509_crt_get_extension(crt, "2.5.29.18", 0,
+						   &prev_der_data,
+						   &critical);
+		if (result < 0
+		    && result != GNUTLS_E_REQUESTED_DATA_NOT_AVAILABLE) {
+			gnutls_assert();
+			return result;
+		}
+	}
+
+	result = _gnutls_encode_othername_data(flags, data, data_size, &encoded_data);
+	if (result < 0) {
+		gnutls_assert();
+		goto finish;
+	}
+
+	/* generate the extension.
+	 */
+	result =
+	    _gnutls_x509_ext_gen_subject_alt_name(GNUTLS_SAN_OTHERNAME, oid,
+	    					  encoded_data.data, encoded_data.size,
+						  &prev_der_data,
+						  &der_data);
+
+	if (result < 0) {
+		gnutls_assert();
+		goto finish;
+	}
+
+	result =
+	    _gnutls_x509_crt_set_extension(crt, "2.5.29.18", &der_data,
+					   critical);
+
+	if (result < 0) {
+		gnutls_assert();
+		goto finish;
+	}
+
+	result = 0;
+
+      finish:
+	_gnutls_free_datum(&der_data);
+	_gnutls_free_datum(&prev_der_data);
+	_gnutls_free_datum(&encoded_data);
+	return result;
+}
+
 /**
  * gnutls_x509_crt_set_proxy:
  * @crt: a certificate of type #gnutls_x509_crt_t
@@ -794,8 +978,6 @@ gnutls_x509_crt_set_proxy(gnutls_x509_crt_t crt,
 		return result;
 	}
 
-	crt->use_extensions = 1;
-
 	return 0;
 }
 
@@ -835,8 +1017,6 @@ gnutls_x509_crt_set_private_key_usage_period(gnutls_x509_crt_t crt,
 
 	_gnutls_free_datum(&der_data);
 
-	crt->use_extensions = 1;
-
  cleanup:
 	return result;
 }
@@ -871,6 +1051,8 @@ gnutls_x509_crt_sign2(gnutls_x509_crt_t crt, gnutls_x509_crt_t issuer,
 		return GNUTLS_E_INVALID_REQUEST;
 	}
 
+	MODIFIED(crt);
+
 	result = gnutls_privkey_init(&privkey);
 	if (result < 0) {
 		gnutls_assert();
@@ -938,6 +1120,8 @@ gnutls_x509_crt_set_activation_time(gnutls_x509_crt_t cert,
 		return GNUTLS_E_INVALID_REQUEST;
 	}
 
+	MODIFIED(cert);
+
 	return _gnutls_x509_set_time(cert->cert,
 				     "tbsCertificate.validity.notBefore",
 				     act_time, 0);
@@ -963,6 +1147,9 @@ gnutls_x509_crt_set_expiration_time(gnutls_x509_crt_t cert,
 		gnutls_assert();
 		return GNUTLS_E_INVALID_REQUEST;
 	}
+
+	MODIFIED(cert);
+
 	return _gnutls_x509_set_time(cert->cert,
 				     "tbsCertificate.validity.notAfter",
 				     exp_time, 0);
@@ -995,6 +1182,8 @@ gnutls_x509_crt_set_serial(gnutls_x509_crt_t cert, const void *serial,
 		return GNUTLS_E_INVALID_REQUEST;
 	}
 
+	MODIFIED(cert);
+
 	ret =
 	    asn1_write_value(cert->cert, "tbsCertificate.serialNumber",
 			     serial, serial_size);
@@ -1031,6 +1220,8 @@ gnutls_x509_crt_set_issuer_unique_id(gnutls_x509_crt_t cert, const void *id,
 		return GNUTLS_E_INVALID_REQUEST;
 	}
 
+	MODIFIED(cert);
+
 	ret =
 	    asn1_write_value(cert->cert, "tbsCertificate.issuerUniqueID",
 			     id, id_size*8);
@@ -1066,6 +1257,8 @@ gnutls_x509_crt_set_subject_unique_id(gnutls_x509_crt_t cert, const void *id,
 		return GNUTLS_E_INVALID_REQUEST;
 	}
 
+	MODIFIED(cert);
+
 	ret =
 	    asn1_write_value(cert->cert, "tbsCertificate.subjectUniqueID",
 			     id, id_size*8);
@@ -1216,7 +1409,6 @@ gnutls_x509_crt_set_crl_dist_points2(gnutls_x509_crt_t crt,
 		goto cleanup;
 	}
 
-	crt->use_extensions = 1;
 	ret = 0;
  cleanup:
 	_gnutls_free_datum(&der_data);
@@ -1273,8 +1465,6 @@ gnutls_x509_crt_cpy_crl_dist_points(gnutls_x509_crt_t dst,
 		return result;
 	}
 
-	dst->use_extensions = 1;
-
 	return 0;
 }
 
@@ -1339,8 +1529,6 @@ gnutls_x509_crt_set_subject_key_id(gnutls_x509_crt_t cert,
 		return result;
 	}
 
-	cert->use_extensions = 1;
-
 	return 0;
 }
 
@@ -1401,8 +1589,6 @@ gnutls_x509_crt_set_authority_key_id(gnutls_x509_crt_t cert,
 		return result;
 	}
 
-	cert->use_extensions = 1;
-
 	return 0;
 }
 
@@ -1472,8 +1658,6 @@ gnutls_x509_crt_set_key_purpose_oid(gnutls_x509_crt_t cert,
 		goto cleanup;
 	}
 
-	cert->use_extensions = 1;
-
 	ret = 0;
  cleanup:
 	_gnutls_free_datum(&der);
@@ -1516,6 +1700,8 @@ gnutls_x509_crt_privkey_sign(gnutls_x509_crt_t crt,
 		return GNUTLS_E_INVALID_REQUEST;
 	}
 
+	MODIFIED(crt);
+
 	/* disable all the unneeded OPTIONAL fields.
 	 */
 	disable_optional_stuff(crt);
@@ -1606,8 +1792,6 @@ gnutls_x509_crt_set_authority_info_access(gnutls_x509_crt_t crt,
 		goto cleanup;
 	}
 
-	crt->use_extensions = 1;
-
       cleanup:
       	if (aia_ctx != NULL)
       		gnutls_x509_aia_deinit(aia_ctx);
@@ -1688,8 +1872,6 @@ gnutls_x509_crt_set_policy(gnutls_x509_crt_t crt,
 	ret = _gnutls_x509_crt_set_extension(crt, "2.5.29.32",
 						&der_data, 0);
 
-	crt->use_extensions = 1;
-
  cleanup:
  	if (policies != NULL)
 	 	gnutls_x509_policies_deinit(policies);
