@@ -83,10 +83,17 @@ static struct cfg_options available_options[] = {
 	{ .name = "dns_name", .type = OPTION_MULTI_LINE },
 	{ .name = "ip_address", .type = OPTION_MULTI_LINE },
 	{ .name = "email", .type = OPTION_MULTI_LINE },
+	{ .name = "krb5_principal", .type = OPTION_MULTI_LINE },
+	{ .name = "other_name", .type = OPTION_MULTI_LINE },
+	{ .name = "other_name_utf8", .type = OPTION_MULTI_LINE },
+	{ .name = "other_name_octet", .type = OPTION_MULTI_LINE },
+	{ .name = "xmpp_name", .type = OPTION_MULTI_LINE },
 	{ .name = "key_purpose_oid", .type = OPTION_MULTI_LINE },
 	{ .name = "nc_exclude_dns", .type = OPTION_MULTI_LINE },
+	{ .name = "nc_exclude_ip", .type = OPTION_MULTI_LINE },
 	{ .name = "nc_exclude_email", .type = OPTION_MULTI_LINE },
 	{ .name = "nc_permit_dns", .type = OPTION_MULTI_LINE },
+	{ .name = "nc_permit_ip", .type = OPTION_MULTI_LINE },
 	{ .name = "nc_permit_email", .type = OPTION_MULTI_LINE },
 	{ .name = "dn_oid", .type = OPTION_MULTI_LINE },
 	{ .name = "crl_dist_points", .type = OPTION_MULTI_LINE },
@@ -156,7 +163,14 @@ typedef struct _cfg_ctx {
 	char **uri;
 	char **ip_addr;
 	char **email;
+	char **krb5_principal;
+	char **other_name;
+	char **other_name_utf8;
+	char **other_name_octet;
+	char **xmpp_name;
 	char **dn_oid;
+	char **permitted_nc_ip;
+	char **excluded_nc_ip;
 	char **permitted_nc_dns;
 	char **excluded_nc_dns;
 	char **permitted_nc_email;
@@ -448,13 +462,20 @@ int template_parse(const char *template)
 	READ_MULTI_LINE("dc", cfg.dc);
 	READ_MULTI_LINE("dns_name", cfg.dns_name);
 	READ_MULTI_LINE("uri", cfg.uri);
+	READ_MULTI_LINE("krb5_principal", cfg.krb5_principal);
+	READ_MULTI_LINE_TOKENIZED("other_name", cfg.other_name);
+	READ_MULTI_LINE_TOKENIZED("other_name_octet", cfg.other_name_octet);
+	READ_MULTI_LINE_TOKENIZED("other_name_utf8", cfg.other_name_utf8);
 
+	READ_MULTI_LINE("xmpp_name", cfg.xmpp_name);
 	READ_MULTI_LINE("ip_address", cfg.ip_addr);
 	READ_MULTI_LINE("email", cfg.email);
 	READ_MULTI_LINE("key_purpose_oid", cfg.key_purpose_oids);
 
+	READ_MULTI_LINE("nc_exclude_ip", cfg.excluded_nc_ip);
 	READ_MULTI_LINE("nc_exclude_dns", cfg.excluded_nc_dns);
 	READ_MULTI_LINE("nc_exclude_email", cfg.excluded_nc_email);
+	READ_MULTI_LINE("nc_permit_ip", cfg.permitted_nc_ip);
 	READ_MULTI_LINE("nc_permit_dns", cfg.permitted_nc_dns);
 	READ_MULTI_LINE("nc_permit_email", cfg.permitted_nc_email);
 
@@ -911,6 +932,103 @@ void get_dn_crt_set(gnutls_x509_crt_t crt)
 	}
 }
 
+static void prefix_to_mask6(unsigned prefix, uint8_t mask[16])
+{
+	int i, j;
+
+	memset(mask, 0, 16);
+
+	for (i = prefix, j = 0; i > 0; i -= 8, j++) {
+		if (i >= 8) {
+			mask[j] = 0xff;
+		} else {
+			mask[j] = (unsigned long)(0xffU << (8 - i));
+		}
+	}
+
+	return;
+}
+
+static void prefix_to_mask4(unsigned prefix, uint8_t _mask[4])
+{
+	uint32_t mask;
+
+	mask = htonl(~((1 << (32 - prefix)) - 1));
+	memcpy(_mask, &mask, 4);
+}
+
+static void mask_ip(uint8_t *ip, uint8_t *mask, unsigned size)
+{
+	unsigned i;
+	for (i=0;i<size;i++)
+		ip[i] &= mask[i];
+}
+
+/* converts an IP to the format RFC5280 expects it */
+static void cidr_to_datum(char *ip, gnutls_datum_t *res)
+{
+	int prefix = -1;
+	uint8_t out[32];
+	int ret;
+	char *p;
+
+	p = strchr(ip, '/');
+	if (p != NULL) {
+		prefix = atoi(p+1);
+		if (prefix == 0) {
+			fprintf(stderr, "Invalid prefix given for %s\n", ip);
+			exit(1);
+		}
+		*p = 0;
+	}
+
+	if (strchr(ip, ':') != 0) { /* IPv6 */
+		if (prefix == -1)
+			prefix = 128;
+
+		if (prefix == 0 || prefix > 128) {
+			fprintf(stderr, "Invalid IPv6 prefix (%d)\n", prefix);
+			exit(1);
+		}
+
+		ret = inet_pton(AF_INET6, ip, out);
+		if (ret == 0) {
+			fprintf(stderr, "Error parsing IPv6 %s\n", ip);
+			exit(1);
+		}
+
+		prefix_to_mask6(prefix, &out[16]);
+		mask_ip(out, &out[16], 16);
+		res->size = 32;
+	} else {
+		if (prefix == -1)
+			prefix = 32;
+
+		if (prefix == 0 || prefix > 32) {
+			fprintf(stderr, "Invalid IPv4 prefix (%d)\n", prefix);
+			exit(1);
+		}
+
+		ret = inet_pton(AF_INET, ip, out);
+		if (ret == 0) {
+			fprintf(stderr, "Error parsing IPv4 %s\n", ip);
+			exit(1);
+		}
+
+		prefix_to_mask4(prefix, &out[4]);
+		mask_ip(out, &out[4], 4);
+		res->size = 8;
+	}
+
+	res->data = malloc(res->size);
+	if (res->data == NULL) {
+		fprintf(stderr, "memory error\n");
+		exit(1);
+	}
+
+	memcpy(res->data, out, res->size);
+}
+
 void crt_constraints_set(gnutls_x509_crt_t crt)
 {
 	int ret;
@@ -920,7 +1038,8 @@ void crt_constraints_set(gnutls_x509_crt_t crt)
 
 	if (batch) {
 		if (cfg.permitted_nc_dns == NULL && cfg.permitted_nc_email == NULL &&
-			cfg.excluded_nc_dns == NULL && cfg.excluded_nc_email == NULL)
+			cfg.excluded_nc_dns == NULL && cfg.excluded_nc_email == NULL &&
+			cfg.permitted_nc_ip == NULL && cfg.excluded_nc_ip == NULL)
 			return; /* nothing to do */
 
 		ret = gnutls_x509_name_constraints_init(&nc);
@@ -929,6 +1048,30 @@ void crt_constraints_set(gnutls_x509_crt_t crt)
 			exit(1);
 		}
 
+		if (cfg.permitted_nc_ip) {
+			for (i = 0; cfg.permitted_nc_ip[i] != NULL; i++) {
+				cidr_to_datum(cfg.permitted_nc_ip[i], &name);
+				ret = gnutls_x509_name_constraints_add_permitted(nc, GNUTLS_SAN_IPADDRESS, &name);
+				if (ret < 0) {
+					fprintf(stderr, "error adding constraint: %s\n", gnutls_strerror(ret));
+					exit(1);
+				}
+				free(name.data);
+			}
+		}
+
+		if (cfg.excluded_nc_ip) {
+			for (i = 0; cfg.excluded_nc_ip[i] != NULL; i++) {
+				cidr_to_datum(cfg.excluded_nc_ip[i], &name);
+				ret = gnutls_x509_name_constraints_add_excluded(nc, GNUTLS_SAN_IPADDRESS, &name);
+				if (ret < 0) {
+					fprintf(stderr, "error adding constraint: %s\n", gnutls_strerror(ret));
+					exit(1);
+				}
+				free(name.data);
+			}
+		}
+
 		if (cfg.permitted_nc_dns) {
 
 			for (i = 0; cfg.permitted_nc_dns[i] != NULL; i++) {
@@ -943,6 +1086,7 @@ void crt_constraints_set(gnutls_x509_crt_t crt)
 			}
 		}
 
+
 		if (cfg.excluded_nc_dns) {
 			for (i = 0; cfg.excluded_nc_dns[i] != NULL; i++) {
 				name.data = (void*)cfg.excluded_nc_dns[i];
@@ -1706,6 +1850,234 @@ void get_dns_name_set(int type, void *crt)
 	}
 }
 
+static int set_krb5_principal(int type, void *crt)
+{
+	int ret = 0, i;
+
+	if (batch) {
+		if (!cfg.krb5_principal)
+			return 0;
+
+		for (i = 0; cfg.krb5_principal[i] != NULL; i ++) {
+			if (type == TYPE_CRT)
+				ret =
+				    gnutls_x509_crt_set_subject_alt_name
+				    (crt, GNUTLS_SAN_OTHERNAME_KRB5PRINCIPAL,
+				     cfg.krb5_principal[i], strlen(cfg.krb5_principal[i]),
+				     GNUTLS_FSAN_APPEND);
+			else
+				ret =
+				    gnutls_x509_crq_set_subject_alt_name
+				    (crt, GNUTLS_SAN_OTHERNAME_KRB5PRINCIPAL,
+				     cfg.krb5_principal[i], strlen(cfg.krb5_principal[i]),
+				     GNUTLS_FSAN_APPEND);
+
+			if (ret < 0)
+				break;
+		}
+	}
+
+	if (ret < 0) {
+		fprintf(stderr, "set_subject_alt_name(GNUTLS_SAN_OTHERNAME_KRB5PRINCIPAL): %s\n",
+			gnutls_strerror(ret));
+		exit(1);
+	}
+
+	return ret;
+}
+
+static int set_othername(int type, void *crt)
+{
+	int ret = 0, i;
+	uint8_t *binname = NULL;
+	unsigned binnamelen = 0;
+	const char *oid;
+
+	if (batch) {
+		if (!cfg.other_name)
+			return 0;
+
+		for (i = 0; cfg.other_name[i] != NULL; i += 2) {
+			oid = cfg.other_name[i];
+
+			if (cfg.other_name[i + 1] == NULL) {
+				fprintf(stderr,
+					"other_name: %s does not have an argument.\n",
+					cfg.other_name[i]);
+				exit(1);
+			}
+
+			HEX_DECODE (cfg.other_name[i+1], binname, binnamelen);
+			if (binnamelen == 0)
+				break;
+
+			if (type == TYPE_CRT)
+				ret =
+				    gnutls_x509_crt_set_subject_alt_othername
+				    (crt, oid,
+				     binname, binnamelen,
+				     GNUTLS_FSAN_APPEND);
+			else
+				ret =
+				    gnutls_x509_crq_set_subject_alt_othername
+				    (crt, oid,
+				     binname, binnamelen,
+				     GNUTLS_FSAN_APPEND);
+			free (binname);
+			binname = NULL;
+
+			if (ret < 0)
+				break;
+		}
+	}
+
+	if (ret < 0) {
+		fprintf(stderr, "set_subject_alt_othername: %s\n",
+			gnutls_strerror(ret));
+		exit(1);
+	}
+
+	return ret;
+}
+
+static int set_othername_utf8(int type, void *crt)
+{
+	int ret = 0, i;
+	const char *oid;
+
+	if (batch) {
+		if (!cfg.other_name_utf8)
+			return 0;
+
+		for (i = 0; cfg.other_name_utf8[i] != NULL; i += 2) {
+			oid = cfg.other_name_utf8[i];
+
+			if (cfg.other_name_utf8[i + 1] == NULL) {
+				fprintf(stderr,
+					"other_name_utf8: %s does not have an argument.\n",
+					cfg.other_name_utf8[i]);
+				exit(1);
+			}
+
+			if (type == TYPE_CRT)
+				ret =
+				    gnutls_x509_crt_set_subject_alt_othername
+				    (crt, oid,
+				     cfg.other_name_utf8[i + 1], strlen(cfg.other_name_utf8[i + 1]),
+				     GNUTLS_FSAN_APPEND|GNUTLS_FSAN_ENCODE_UTF8_STRING);
+			else
+				ret =
+				    gnutls_x509_crq_set_subject_alt_othername
+				    (crt, oid,
+				     cfg.other_name_utf8[i + 1], strlen(cfg.other_name_utf8[i + 1]),
+				     GNUTLS_FSAN_APPEND|GNUTLS_FSAN_ENCODE_UTF8_STRING);
+
+			if (ret < 0)
+				break;
+		}
+	}
+
+	if (ret < 0) {
+		fprintf(stderr, "set_subject_alt_othername: %s\n",
+			gnutls_strerror(ret));
+		exit(1);
+	}
+
+	return ret;
+}
+
+static int set_othername_octet(int type, void *crt)
+{
+	int ret = 0, i;
+	const char *oid;
+
+	if (batch) {
+		if (!cfg.other_name_octet)
+			return 0;
+
+		for (i = 0; cfg.other_name_octet[i] != NULL; i += 2) {
+			oid = cfg.other_name_octet[i];
+
+			if (cfg.other_name_octet[i + 1] == NULL) {
+				fprintf(stderr,
+					"other_name_octet: %s does not have an argument.\n",
+					cfg.other_name_octet[i]);
+				exit(1);
+			}
+
+			if (type == TYPE_CRT)
+				ret =
+				    gnutls_x509_crt_set_subject_alt_othername
+				    (crt, oid,
+				     cfg.other_name_octet[i + 1], strlen(cfg.other_name_octet[i + 1]),
+				     GNUTLS_FSAN_APPEND|GNUTLS_FSAN_ENCODE_OCTET_STRING);
+			else
+				ret =
+				    gnutls_x509_crq_set_subject_alt_othername
+				    (crt, oid,
+				     cfg.other_name_octet[i + 1], strlen(cfg.other_name_octet[i + 1]),
+				     GNUTLS_FSAN_APPEND|GNUTLS_FSAN_ENCODE_OCTET_STRING);
+
+			if (ret < 0)
+				break;
+		}
+	}
+
+	if (ret < 0) {
+		fprintf(stderr, "set_subject_alt_othername: %s\n",
+			gnutls_strerror(ret));
+		exit(1);
+	}
+
+	return ret;
+}
+
+static int set_xmpp_name(int type, void *crt)
+{
+	int ret = 0, i;
+
+	if (batch) {
+		if (!cfg.xmpp_name)
+			return 0;
+
+		for (i = 0; cfg.xmpp_name[i] != NULL; i ++) {
+			if (type == TYPE_CRT)
+				ret =
+				    gnutls_x509_crt_set_subject_alt_name
+				    (crt, GNUTLS_SAN_OTHERNAME_XMPP,
+				     cfg.xmpp_name[i], strlen(cfg.xmpp_name[i]),
+				     GNUTLS_FSAN_APPEND);
+			else
+				ret =
+				    gnutls_x509_crq_set_subject_alt_name
+				    (crt, GNUTLS_SAN_OTHERNAME_XMPP,
+				     cfg.xmpp_name[i], strlen(cfg.xmpp_name[i]),
+				     GNUTLS_FSAN_APPEND);
+
+			if (ret < 0)
+				break;
+		}
+	}
+
+	if (ret < 0) {
+		fprintf(stderr, "set_subject_alt_name(XMPP): %s\n",
+			gnutls_strerror(ret));
+		exit(1);
+	}
+
+	return ret;
+}
+
+
+void get_other_name_set(int type, void *crt)
+{
+	set_othername(type, crt);
+	set_othername_octet(type, crt);
+	set_othername_utf8(type, crt);
+	set_xmpp_name(type, crt);
+	set_krb5_principal(type, crt);
+}
+
 void get_policy_set(gnutls_x509_crt_t crt)
 {
 	int ret = 0, i;
@@ -1815,7 +2187,7 @@ int get_sign_status(int server)
 	} else {
 		if (server)
 			msg =
-			    "Will the certificate be used for signing (DHE and RSA-EXPORT ciphersuites)? (Y/n): ";
+			    "Will the certificate be used for signing (DHE ciphersuites)? (Y/n): ";
 		else
 			msg =
 			    "Will the certificate be used for signing (required for TLS)? (Y/n): ";
