@@ -1,7 +1,6 @@
 /*
- * Copyright (C) 2000-2016 Free Software Foundation, Inc.
- * Copyright (C) 2013-2016 Nikos Mavrogiannopoulos
- * Copyright (C) 2015-2016 Red Hat, Inc.
+ * Copyright (C) 2000-2014 Free Software Foundation, Inc.
+ * Copyright (C) 2013-2014 Nikos Mavrogiannopoulos
  *
  * This file is part of GnuTLS.
  *
@@ -74,7 +73,7 @@
 int resume, starttls, insecure, ranges, rehandshake, udp, mtu,
     inline_commands;
 const char *hostname = NULL;
-const char *service = NULL;
+char service[32]="";
 int record_max_size;
 int fingerprint;
 int crlf;
@@ -105,10 +104,6 @@ static gnutls_psk_client_credentials_t psk_cred;
 static gnutls_anon_client_credentials_t anon_cred;
 static gnutls_certificate_credentials_t xcred;
 
-/* The number of seconds we wait for a reply from peer prior to
- * closing the connection. */
-#define TERM_TIMEOUT 8000
-
 /* end of global stuff */
 
 /* prototypes */
@@ -422,9 +417,7 @@ static int cert_verify_callback(gnutls_session_t session)
 	unsigned int status = 0;
 	int ssh = ENABLED_OPT(TOFU);
 	int strictssh = ENABLED_OPT(STRICT_TOFU);
-#ifdef HAVE_DANE
 	int dane = ENABLED_OPT(DANE);
-#endif
 	int ca_verify = ENABLED_OPT(CA_VERIFICATION);
 	const char *txt_service;
 
@@ -463,8 +456,8 @@ static int cert_verify_callback(gnutls_session_t session)
 		}
 	}
 
-#ifdef HAVE_DANE
 	if (dane) {		/* try DANE auth */
+#ifdef HAVE_DANE
 		int port;
 		unsigned vflags = 0;
 		unsigned int sflags =
@@ -502,9 +495,12 @@ static int cert_verify_callback(gnutls_session_t session)
 			if (status != 0 && !insecure && !ssh)
 				return -1;
 		}
-
-	}
+#else
+		fprintf(stderr, "*** DANE error: GnuTLS is not compiled with DANE support.\n");
+		if (!insecure && !ssh)
+			return -1;
 #endif
+	}
 
 	if (ssh) {		/* try ssh auth */
 		unsigned int list_size;
@@ -834,7 +830,7 @@ static int check_net_or_keyboard_input(socket_st * hd)
 #endif
 
 		tv.tv_sec = 0;
-		tv.tv_usec = 500 * 1000;
+		tv.tv_usec = 50 * 1000;
 
 		if (hd->secure == 1)
 			if (gnutls_record_check_pending(hd->session))
@@ -885,17 +881,20 @@ static int try_rehandshake(socket_st * hd)
 static int try_resume(socket_st * hd)
 {
 	int ret;
+	gnutls_datum_t rdata = {NULL, 0};
 
-	char *session_data;
-	size_t session_data_size = 0;
-
-	gnutls_session_get_data(hd->session, NULL, &session_data_size);
-	session_data = (char *) malloc(session_data_size);
-	if (session_data == NULL)
-		return GNUTLS_E_MEMORY_ERROR;
-
-	gnutls_session_get_data(hd->session, session_data,
-				&session_data_size);
+	if (gnutls_session_is_resumed(hd->session) == 0) {
+		/* not resumed - obtain the session data */
+		ret = gnutls_session_get_data2(hd->session, &rdata);
+		if (ret < 0) {
+			rdata.data = NULL;
+		}
+	} else {
+		/* resumed - try to reuse the previous session data */
+		rdata.data = hd->rdata.data;
+		rdata.size = hd->rdata.size;
+		hd->rdata.data = NULL;
+	}
 
 	printf("- Disconnecting\n");
 	socket_bye(hd);
@@ -908,9 +907,12 @@ static int try_resume(socket_st * hd)
 	        socket_starttls(hd, OPT_ARG(STARTTLS_PROTO));
 
 	hd->session = init_tls_session(hostname);
-	gnutls_session_set_data(hd->session, session_data,
-				session_data_size);
-	free(session_data);
+	if (rdata.data) {
+		hd->rdata.data = rdata.data;
+		hd->rdata.size = rdata.size;
+
+		gnutls_session_set_data(hd->session, hd->rdata.data, hd->rdata.size);
+	}
 
 	ret = do_handshake(hd);
 	if (ret < 0) {
@@ -1146,26 +1148,6 @@ print_other_info(gnutls_session_t session)
 	}
 }
 
-static void flush_socket(socket_st *hd, unsigned ms)
-{
-	int ret, ii;
-	char buffer[MAX_BUF + 1];
-
-	memset(buffer, 0, MAX_BUF + 1);
-	ret = socket_recv_timeout(hd, buffer, MAX_BUF, ms);
-	if (ret == 0)
-		return;
-	else if (ret > 0) {
-		if (verbose != 0)
-			printf("- Received[%d]: ", ret);
-
-		for (ii = 0; ii < ret; ii++) {
-			fputc(buffer[ii], stdout);
-		}
-		fflush(stdout);
-	}
-}
-
 int main(int argc, char **argv)
 {
 	int ret;
@@ -1176,7 +1158,6 @@ int main(int argc, char **argv)
 	ssize_t bytes, keyboard_bytes;
 	char *keyboard_buffer_ptr;
 	inline_cmds_st inline_cmds;
-	unsigned last_op_is_write = 0;
 #ifndef _WIN32
 	struct sigaction new_action;
 #endif
@@ -1279,7 +1260,6 @@ int main(int argc, char **argv)
 
 		if (inp == IN_NET) {
 			memset(buffer, 0, MAX_BUF + 1);
-			last_op_is_write = 0;
 			ret = socket_recv(&hd, buffer, MAX_BUF);
 
 			if (ret == 0) {
@@ -1325,8 +1305,6 @@ int main(int argc, char **argv)
 						break;
 					}
 				} else {
-					if (last_op_is_write)
-						flush_socket(&hd, TERM_TIMEOUT);
 					user_term = 1;
 					break;
 				}
@@ -1366,7 +1344,6 @@ int main(int argc, char **argv)
 				}
 			}
 
-			last_op_is_write = 1;
 			if (ranges
 			    && gnutls_record_can_use_length_hiding(hd.
 								   session))
@@ -1556,12 +1533,12 @@ static void cmd_parser(int argc, char **argv)
 	mtu = OPT_VALUE_MTU;
 
 	if (HAVE_OPT(PORT)) {
-		service = OPT_ARG(PORT);
+		snprintf(service, sizeof(service), "%s", OPT_ARG(PORT));
 	} else {
 		if (HAVE_OPT(STARTTLS_PROTO))
-			service = starttls_proto_to_service(OPT_ARG(STARTTLS_PROTO));
+			snprintf(service, sizeof(service), "%s", starttls_proto_to_service(OPT_ARG(STARTTLS_PROTO)));
 		else
-			service = "443";
+			strcpy(service, "443");
 	}
 
 	record_max_size = OPT_VALUE_RECORDSIZE;
@@ -1973,5 +1950,5 @@ cleanup:
 
 	if (failed > 0)
 		return -1;
-	return ok >= 1 ? (int) ok : -1;
+	return ok > 1 ? (int) ok : -1;
 }
