@@ -10,12 +10,13 @@ typedef struct {
 	struct addrinfo *ptr;
 	struct addrinfo *addr_info;
 	int verbose;
+
+	/* resumption data */
+	gnutls_datum_t rdata;
 } socket_st;
 
 ssize_t socket_recv(const socket_st * socket, void *buffer,
 		    int buffer_size);
-ssize_t socket_recv_timeout(const socket_st * socket, void *buffer,
-		    int buffer_size, unsigned ms);
 ssize_t socket_send(const socket_st * socket, const void *buffer,
 		    int buffer_size);
 ssize_t socket_send_range(const socket_st * socket, const void *buffer,
