@@ -1,8 +1,7 @@
 /*
- * Copyright (C) 2004-2016 Free Software Foundation, Inc.
- * Copyright (C) 2016 Red Hat, Inc.
+ * Copyright (C) 2004-2012 Free Software Foundation, Inc.
  *
- * Author: Simon Josefsson, Nikos Mavrogiannopoulos
+ * Author: Simon Josefsson
  *
  * This file is part of GnuTLS.
  *
@@ -63,16 +62,12 @@ extern void fail_ignore(const char *format, ...)
 extern void success(const char *format, ...)
     __attribute__ ((format(printf, 1, 2)));
 
-extern void c_print(const unsigned char *str, size_t len);
 extern void escapeprint(const char *str, size_t len);
 extern void hexprint(const void *str, size_t len);
 extern void binprint(const void *str, size_t len);
 int disable_system_calls(void);
 void sec_sleep(int sec);
 
-void test_cli_serv(gnutls_certificate_credentials_t server_cred, const char *prio,
-              const gnutls_datum_t *ca_cert, const char *host);
-
 #define TMPNAME_SIZE 128
 char *get_tmpname(char s[TMPNAME_SIZE]);
 
